/*********************************************
*
*	ClineDiseqParamRec.cpp
*
*	This base class holds a set of parameter values that
*	are used in a metropolis algorithm for numerically 
*	maximizing an estimate of gametic phase disequilibrium D.
*
*	Always use this in combination with a cline shape class,
*	and always derive functions to vary these values and check
*	their limits.
*
*********************************************/
#ifndef ClineDiseqParamReccpp
#define ClineDiseqParamReccpp = 1

#include "ClineDiseqParamRec.h"
#include "StringClass.h"

	
ParamRec* ClineDiseqParamRec::Duplicate(void){
	return NEW ClineDiseqParamRec(*this);
	}//end ClineDiseqParamRec::Duplicate()
	
//void ClineDiseqParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineDiseqParamRec"<<endl;
//  	}//end

long ClineDiseqParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end



int ClineDiseqParamRec::ChangeD(Ran3& ran,double sigma,
									ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary D_
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldD = D_;
	do{
		D_ = ran.NormDev(oldD, sigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end ClineDiseqParamRec::ChangeD(Ran3&, double, ParamRec*, ParamRec*)





long double ClineDiseqParamRec::ClineDiseqLikelihood (ClineGeneticData* alloData){
	//sample-size based weighting removed
	long nSample;
	long double lnL=0.0;
	long double adjustedD = 0.0;
	aString blank;blank.AppendCharToLen(' ',alloData->alleleLen());
	aMatrix<double>* ff = ZeroMatrix((double)0,2,2);
	aMatrix<double>* bigXX = ZeroMatrix((double)0,2,2);
	aMatrix<double>& f = *ff;
	aMatrix<double>& bigX = *bigXX;
	aMatrix<double>* bigNN = ZeroMatrix((double)0,3,3);
	aMatrix<double>& bigN = *bigNN;
	int adjByFis = true;//set to false here if pops are in HW proportions
    double FisCenter = 0.0;
    if (adjByFis){//calculate Fis for the center pops only
//		FstatisticAnalysis centerPopData(*alloData,alloData->centerPopList_);
//		centerPopData.WeirTheta();
//		FisCenter = centerPopData.Fis(FstatisticAnalysis::WandC);
		FisCenter = alloData->CenterPopFis();
		}
	for(int popNum=0;popNum<alloData->centerPopList_.len();++popNum){
		int pop = alloData->centerPopList_[popNum];
		long double x = alloData->Locality(pop);//this location
		aList<long double> coords1, coords2; coords1.Append(x);coords2.Append(x);
		for(int loc1=0;loc1<alloData->loci()-1;++loc1){
			coords1.Append(loc1);
			long double pAtCline1 = ExpectedValue(coords1);
			pAtCline1 = MAX(pAtCline1,(long double)10.0e-5);//eliminates 0 expected values in likelihood
			pAtCline1 = MIN(pAtCline1,(long double)(1.0-10e-5));
			StringList& alleleNamesList1 = alloData->allelesPresent[loc1];
			for(int loc2 = (loc1 + 1); loc2<alloData->loci();++loc2){
				coords2.Append(loc2);
				long double pAtCline2 = ExpectedValue(coords2);
				pAtCline2 = MAX(pAtCline2,(long double)10.0e-5);//eliminates 0 expected values in likelihood
				pAtCline2 = MIN(pAtCline2,(long double)(1.0-10e-5));
		       	adjustedD = (D_ / sqrt(pAtCline1 * (1.0 - pAtCline1) * pAtCline2 * (1.0 - pAtCline2))) * (1.0 + FisCenter);
				adjustedD = MIN(adjustedD,(long double)1.0);//at low expected cline freqs, adjustedD can exceed 1.0
				adjustedD = MAX(adjustedD,(long double)-1.0);//and this is impossible


				StringList& alleleNamesList2 = alloData->allelesPresent[loc2];
				for(int allele1=0;allele1<alleleNamesList1.len();++allele1){
					//get freq of allele1 (at loc1) in pop
					double freq1 = alloData->GetFreq(pop, loc1, allele1);
					if ((freq1 > 0.0) && (freq1 < 1.0)){//otherwise D is undefined
						aString& alleleName1 = alleleNamesList1[allele1];
						for(int allele2=0;allele2<alleleNamesList2.len();++allele2){
							nSample = 0;//set sample size to zero, to count for blanks
							//get freq of allele2 (at loc2) in pop
							double freq2 = alloData->GetFreq(pop, loc2, allele2);
							if ((freq2 > 0.0) && (freq2 < 1.0)){//otherwise D is undefined
								f[0][0] = freq1 * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[0][1] = freq1 * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][0] = (1.0 - freq1) * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][1] = (1.0 - freq1) * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								//sometimes these values can be less than zero due to sampling error
								//implying that it is infinitely improbable that an observed value
								//could occur under the model.
								//therefore, values <=0 are set to a very small number, and will
								//give a very low finite likelihood
								for(int ind1=0;ind1<2;++ind1){
									for(int ind2=0;ind2<2;++ind2){
										if(f[ind1][ind2] <= 0.0){
											f[ind1][ind2] = 1.0e-10;}
										}}

								//get genotype strings for each individual and  read joint frequencies
								aString& alleleName2 = alleleNamesList2[allele2];
//								aMatrix<double>* bigNN = ZeroMatrix((double)0,3,3);
//								aMatrix<double>& bigN = *bigNN;
								bigN[0][0]=bigN[0][1]=bigN[0][2]=bigN[1][0]=bigN[1][1]=bigN[1][2]=bigN[2][0]=bigN[2][1]=bigN[2][2]=0.0;
 								int sizePop = alloData->individuals(pop);
								for(int i=0;i<sizePop;++i){
//									aString* spot1 = alloData->GetGenotypeString(pop,i,loc1);
									StringList spot1 = alloData->Genotype(pop,i,loc1);
									//make sure it doesn't have any blanks
									int blankAt1 = false;
//       								if(spot1->len() < 2){
       								if(spot1.len() < 2){
       										blankAt1 = true;//happens sometimes at end of data line
											}//with missing loci
										else{
//											if(((*spot1)[0] == ' ') || ((*spot1)[1] == ' ')){
											if((spot1[0] == blank) || (spot1[1] == blank)){
												blankAt1 = true;}
											}
									if(!blankAt1){
//										aString* spot2 = alloData->GetGenotypeString(pop,i,loc2);
										StringList spot2 = alloData->Genotype(pop,i,loc2);
										//make sure it doesn' t have any blanks
										int blankAt2 = false;
//       									if(spot2->len() < 2){
       									if(spot2.len() < 2){
       											blankAt2 = true;//happens sometimes at end of data line
												}//with missing loci
											else{
//												if(((*spot2)[0] == ' ') || ((*spot2)[1] == ' ')){
												if((spot2[0] == blank) || (spot2[1] == blank)){
													blankAt2 = true;}
												}
										if(!blankAt2){
											++nSample;
											//use this indiv in calculation
											//count instances of target alleles at each locus in this indiv
											int i1 = 2;
											for(int chrom=0;chrom<2;++chrom){
//												aString alleleVal1((*spot1)[chrom]);
												aString alleleVal1(spot1[chrom]);
												if( alleleName1 == alleleVal1){
													--i1;}}
											int i2 = 2;
											for(int chrom=0;chrom<2;++chrom){
//												aString alleleVal2((*spot2)[chrom]);
												aString alleleVal2(spot2[chrom]);
												if(alleleName2 == alleleVal2){
													--i2;}}
											bigN[i1][i2]++;
											}//end if not blankAt2
//										delete spot2; spot2=0;
										}//end if not blankAt1
//									delete spot1; spot1=0;
									}//end for i
								//calculate Hill's (1974) intermediate variables & iterative method
								bigX[0][0] = 2.0 * bigN[0][0] + bigN[0][1] + bigN[1][0];
								bigX[1][0] = 2.0 * bigN[2][0] + bigN[1][0] + bigN[2][1];
								bigX[0][1] = 2.0 * bigN[0][2] + bigN[0][1] + bigN[1][2];
								bigX[1][1] = 2.0 * bigN[2][2] + bigN[1][2] + bigN[2][1];
								for(int index1=0;index1<2;++index1){
									for(int index2=0;index2<2;++index2){
										if (f[index1][index2] > 0.0){
											lnL += bigX[index1][index2] * ln(f[index1][index2]);
											}}}//end for both indexes
								lnL += bigN[1][1] * ln(f[0][0] * f[1][1] + f[0][1] * f[1][0]);
//								delete bigNN; bigNN=0;
								}//end if 0<freq2<1
							}//end for allele2
						}//end if 0<freq1<1
					}//end for allele1
				coords2.DeleteLast();
				}//end for loc2
			coords1.DeleteLast();
			}//end for loc1
		}//end for popNum
	delete ff;ff=0;
	delete bigXX;bigXX=0;
	delete bigNN; bigNN=0;
    lnLDiseq(lnL);
    return lnL;
	}//end function ClineDiseqParamRec::ClineDiseqLikelihood


long double ClineDiseqParamRec::D(void){
	return D_;
	}//end ClineDiseqParamRec::D( )


void ClineDiseqParamRec::D(long double dd){
	D_ = dd;
	}//end ClineDiseqParamRec::D( )


void ClineDiseqParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	D_= i.D_;
	lnLDiseq_ = 0.0;
	}//end ClineDiseqParamRec::Initialize(ClineFitInitializer&)


long double ClineDiseqParamRec::lnLDiseq(void){
		return lnLDiseq_;}

		
void ClineDiseqParamRec::lnLDiseq(long double lnLval){
		lnLDiseq_ = lnLval;}
		
		
void ClineDiseqParamRec::MultiplyParam(int /*paramNum*/, long double factor){
	//use to rescale a param deviation
	D_ *= factor;
	}//ClineDiseqParamRec::MultiplyParam(int, double)


int ClineDiseqParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
//	ClineDiseqParamRec* lower = (ClineDiseqParamRec*) lowerLimits->pointerToDerivedSelf;
//	ClineDiseqParamRec* upper = (ClineDiseqParamRec*) upperLimits->pointerToDerivedSelf;
	ClineDiseqParamRec* lower = (ClineDiseqParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineDiseqParamRec* upper = (ClineDiseqParamRec*) upperLimits->GetPointerToDerivedSelf();
	if((D_<lower->D_) || (D_>upper->D_)){return 0;}
	return 1;
	}//ClineDiseqParamRec::MeetsParamCriteria (ParamRec*, ParamRec*)


int ClineDiseqParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	//override in derived classes
	return 0;
	}//ClineDiseqParamRec::NumDerivedParams()


int ClineDiseqParamRec::NumParams(void){
	//number of parameters that could vary
	return 1;
	}//ClineDiseqParamRec::NumParams()


aString* ClineDiseqParamRec::ParamName(int /*paramNum*/){
	//returns value of parameter
	//shouldn't arrive here; use derived classes
	return NEW aString("D");
	}//end ClineDiseqParamRec::ParamName(int )



void ClineDiseqParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline,sepchar);
	dataline.cat(sepchar);
	dataline.cat(lnLDiseq_,2,15);
	dataline.cat(sepchar);
	dataline.cat(D_,2,15);
	}//end ClineDiseqParamRec::ParamString(aString&, char)


long double ClineDiseqParamRec::ParamValue(int /*paramNum*/){
	return D_;
	}//end ClineDiseqParamRec::ParamValue(int )



void ClineDiseqParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outStr("D: ");
	outStr.cat(D_,2,9);
	outStr.cat ("  lnLDiseq: ");
	outStr.cat(lnLDiseq_,6,9);
	outFile<<outStr<<endl;
	}//ClineDiseqParamRec::Print(ostream&, long, aString&)



void ClineDiseqParamRec::SetParamValue(int /*paramNum*/, long double newVal){
	//sets value of parameter
	D_ = newVal;
	}//end ClineDiseqParamRec::SetParamValue(int,long double)



void ClineDiseqParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the column headings that will be printed to a big file
	ParamRec::ParamString(dataline,sepchar);
	dataline.cat(sepchar);
	dataline.cat("lnLDiseq");
	dataline.cat(sepchar);
	dataline.cat("D");
	}//end ClineDiseqParamRec::TitleString(aString&, char)


void ClineDiseqParamRec::operator=(ParamRec& p){
//	ClineDiseqParamRec* pp = (ClineDiseqParamRec*) p.pointerToDerivedSelf;
	ClineDiseqParamRec* pp = (ClineDiseqParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;
	D_ = pp->D_;
	lnLDiseq_ = pp->lnLDiseq_;	
	}//end  ClineDiseqParamRec::operator=(ParamRec&)


int	operator==(ClineDiseqParamRec& p, ClineDiseqParamRec& q){
	return ((p.lnL_ == q.lnL_) &&(p.D_ == q.D_)&&(p.lnLDiseq_ == q.lnLDiseq_));
	}//end operator==(ClineDiseqParamRec&, ClineDiseqParamRec&)


int	operator!=(ClineDiseqParamRec& p, ClineDiseqParamRec& q){
	return !(p==q);
	}//end operator!=(ClineDiseqParamRec&, ClineDiseqParamRec&)




#endif


