/***************************************************
*
*	ClineFitFunctions.cpp
*
*	These likelihood functions fit parameters to cline data.
*	Feed them derived ParamRec classes that contain cline parameters
*	and the appropriate corresponding derived MetropolisHastingsData 
*	classes that contain clinal data.
*
***************************************************/


#ifndef ClineFitFunctionscpp
#define ClineFitFunctionscpp = 1


#include "ClineFitFunctions.h"


long double MaxLikeDevFromCline (ParamRec& p, MetropolisHastingsData* data){
	ClineData* theData = (ClineData*) data;
 	long double runningTotal = 0.0;
	if((theData->dataType[0]=="codominant")||(theData->dataType[0]=="haplotype")){
		 	for(int tr=0;tr<theData->NumTraits();++tr){//loci estimated separately
  				runningTotal += MaxLikeDevFromClinePerHIData(p, theData, tr);
  				}//tr
   			}//end if
		else if(theData->dataType[0]=="morph"){//all traits estimated simultaneously
   			runningTotal += MaxLikeDevFromClineQuantitative2(p, theData);
   			}//end if
   	p.lnL(runningTotal);
 	return runningTotal;
	}//end MaxLikeDevFromCline (ParamRec&, MetropolisHastingsData*)



long double MaxLikeDevFromClinePerHIData (ParamRec& p, MetropolisHastingsData* data, int loc){
	ClineGeneticData* theData = (ClineGeneticData*) data;
	long double lnL = 0.0;
	for(int i=0;i<theData->HybridIndicesByLocus.len();++i){
    	double obs = theData->HybridIndicesByLocus[i][loc].hi();
    	double x = theData->HybridIndicesByLocus[i][loc].x();
    	aList<long double> coords;  coords.Append(x); coords.Append(loc);
//    	aList<double> coords;  coords.Append(x); coords.Append(0);//use for pre-200304 Cline param recs
    	long double expected = p.ExpectedValue(coords);
    	expected = MAX(expected, (long double)(10.0e-5));//bounds lower limit to logit transform equivalent
    	expected = MIN(expected, (long double)(1.0-10.0e-5));//bounds upper limit to logit transform equivalent
     	lnL += obs * ln(expected);
     	lnL += (1.0 - obs) * ln(1.0 - expected);
   		}//end for i
	p.lnL(lnL);
	return lnL;
	}//end MaxLikeDevFromClinePerHIData (ParamRec&, MetropolisHastingsData*, int)



long double MaxLikeDevFromClineQuantitative (ParamRec& p, MetropolisHastingsData* data, int trait){
	ClinePhenotypicData* theData = (ClinePhenotypicData*) data;
	long double lnL = 0.0;
	aList<long double> coords;  
	//body here
	for(int pop=0;pop<theData->npop();++pop){
		long double x = theData->Locality(pop);
    	coords.Append(x); coords.Append(trait);
		long double mu = p.ExpectedValue(coords);//this function appends the phenotypic variance to coords
		long double Vp = coords.Last();
		if(Vp>0.0){
			long double nObs = theData->popTraitStats[pop][trait].n();
			long double meanObs = theData->popTraitStats[pop][trait].mean();
			long double varObs = theData->popTraitStats[pop][trait].var();
			lnL += - (nObs * log(Vp)/2.0) - (nObs * (varObs + (meanObs - mu)*(meanObs-mu))/(2.0*Vp));//eqn 3.4.2 in Edwards 1991
			}//if
			else{//make zero variance highly unlikely to avoid a singularity during fitting
				lnL -= 1.0e100;}
		coords.Clear();
		}//p
	p.lnL(lnL);
	return lnL;
	}//end MaxLikeDevFromClineQuantitative (ParamRec&, MetropolisHastingsData*, int)



long double MaxLikeDevFromClineQuantitative2 (ParamRec& p, MetropolisHastingsData* data){
	ClinePhenotypicData* theData = (ClinePhenotypicData*) data;
	ClineShapeCWTlTrTBQ2ParamRec* pp = (ClineShapeCWTlTrTBQ2ParamRec*) p.GetPointerToDerivedSelf();
	long double lnL = 0.0;
	//body here
	for(int pop=0;pop<theData->npop();++pop){
		long double x = theData->Locality(pop);
		int region = pop;
		if(pp->covMatrixListType_<=3){
			region = theData->LocationRelativeToCenterList(x);}
		aMatrix<long double>* invCov_z = pp->InverseCovTraitMatrix(region);//calc & store in paramrec
		if(invCov_z != NULL){//if invCov_z exists
			aVector<long double>* mu_z = pp->ZonCline(x);//assumes no missing values
			mu_z->Multiply(-1.0);
			long double sigDet = pp->CovTraitDeterminant(region);//calc & store in paramrec
			long double r = theData->NumTraits();
			long n = theData->popSize[pop];
			lnL += -(n*r/2.0)*ln(2.0*PI()) - n*ln(sigDet)/2.0;
			for(int i=0;i<n;++i){
				aVector<long double>* z = theData->traitVector(pop,i);
				aVector<long double>* zd = z->Add(*mu_z);
				aMatrix<long double>* zdev1 = NEW aMatrix<long double>(); 
				zdev1->ConcatRow(*zd);//convert to matrix form
				aMatrix<long double>* zdev2 = NEW aMatrix<long double>(); 
				zdev2->ConcatCol(*zd);//convert to matrix form
				aMatrix<long double>* zdevInvCov = zdev1->Multiply(*invCov_z);
				aMatrix<long double>* zdevInvCovZdev = zdevInvCov->Multiply(*zdev2);
				long double expterm = (*zdevInvCovZdev)[0][0];
				if(expterm < (long double)0.0){//bug someplace; catch it here with impossibly low likelihood
					lnL = (long double)-1.0e100;
					p.lnL(lnL);
					delete invCov_z; 		invCov_z=0;
					delete z;  				z=0;
					delete zd;  			zd=0;
					delete zdev1;  			zdev1=0;
					delete zdev2;  			zdev2=0;
					delete zdevInvCov;  	zdevInvCov=0;
					delete zdevInvCovZdev;  zdevInvCovZdev=0;
					return lnL;
					}
				lnL += - expterm;
				delete z;  				z=0;
				delete zd;  			zd=0;
				delete zdev1;  			zdev1=0;
				delete zdev2;  			zdev2=0;
				delete zdevInvCov;  	zdevInvCov=0;
				delete zdevInvCovZdev;  zdevInvCovZdev=0;
				}
			delete mu_z;  mu_z=0;
			}//if
			else{//make zeroed inverse covariance matrix highly unlikely to avoid a singularity during fitting
				lnL = (long double)-1.0e100;}
		delete invCov_z; invCov_z=0;
		}//p
	p.lnL(lnL);
	if(lnL>0.0){
		aError("Bug: lnL >0.0");
		for(int r=0;r<pp->regions();++r){
			pp->covMatrixList_[r].Print();
			cout<<"det: "<<pp->covMatrixList_[r].Determinant()<<endl;}
		}
	return lnL;
	}//end MaxLikeDevFromClineQuantitative2 (ParamRec&, MetropolisHastingsData*, int)



long double MaxLikeDevFromClineAndDiseq6 (ParamRec& p, MetropolisHastingsData* data){
	//use to fit 6-parameter clines with disequilibrium estimation
	long double runningTotal = MaxLikeDevFromCline(p,data);
	ClineShapeCWTlTrDParamRec* pp = (ClineShapeCWTlTrDParamRec*) p.GetPointerToDerivedSelf();
	ClineGeneticData* alloData = (ClineGeneticData*) data;
	long double lnLdiseq = pp->ClineDiseqLikelihood (alloData);
	pp->lnL(runningTotal+lnLdiseq);
 	return runningTotal + lnLdiseq;
	}//end MaxLikeDevFromCline (ParamRec&, MetropolisHastingsData*)




long double MaxLikeDevFromClineAndDiseq7 (ParamRec& p, MetropolisHastingsData* data){
	//use to fit 6-parameter clines with disequilibrium estimation
	long double runningTotal = MaxLikeDevFromCline(p,data);
	ClineMultiCWTlTrDParamRec* pp = (ClineMultiCWTlTrDParamRec*) p.GetPointerToDerivedSelf();
	ClineGeneticData* alloData = (ClineGeneticData*) data;
	long double lnLdiseq = pp->ClineDiseqLikelihood (alloData);
	pp->lnL(runningTotal+lnLdiseq);
 	return runningTotal + lnLdiseq;
	}//end MaxLikeDevFromCline (ParamRec&, MetropolisHastingsData*)




long double MaxLikeDevFromClineAndDiseq9 (ParamRec& p, MetropolisHastingsData* data){
	//use to fit 6-parameter clines with disequilibrium estimation
	long double runningTotal = 0.0;
//	ClineShapeCWTlTrTBDParamRec* pp = (ClineShapeCWTlTrTBDParamRec*) p.GetPointerToDerivedSelf();
	ClineMultiCWTlTrTBDParamRec* pp = (ClineMultiCWTlTrTBDParamRec*) p.GetPointerToDerivedSelf();
	ClineGeneticData* alloData = (ClineGeneticData*) data;
	for(int loc=0;loc<alloData->loci();++loc){
		runningTotal += MaxLikeDevFromClinePerHIData (pp->traitParamSet_[loc], data, loc);
		}
	long double lnLdiseq = pp->ClineDiseqLikelihood (alloData);
	pp->lnL(runningTotal+lnLdiseq);
 	return runningTotal + lnLdiseq;
	}//end MaxLikeDevFromCline (ParamRec&, MetropolisHastingsData*)

















#endif
