/******************************************************
*
*	ClineMultiQuantTraitParamRec.cpp
*
*	Parameters for a multi-trait, 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL, zR, and left and right quantitative trait values.  The middle 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.  Disequilibrium
*	is also estimated and derived parameters calculated.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineMultiQuantTraitParamReccpp
#define ClineMultiQuantTraitParamReccpp = 1

#include "ClineMultiQuantTraitParamRec.h"

	
ParamRec* ClineMultiQuantTraitParamRec::Duplicate(void){
	ClineMultiQuantTraitParamRec* copy = NEW ClineMultiQuantTraitParamRec(*this);
	return copy;
	}//end ClineMultiQuantTraitParamRec::Duplicate()


ClineMultiQuantTraitParamRec* ClineMultiQuantTraitParamRec::_Duplicate(void){
	ClineMultiQuantTraitParamRec* copy = NEW ClineMultiQuantTraitParamRec(*this);
	return copy;
	}//end ClineMultiQuantTraitParamRec::_Duplicate()


void ClineMultiQuantTraitParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineMultiQuantTraitParamRec"<<endl;
  	}//end





double ClineMultiQuantTraitParamRec::aveWidth(void){
	//average width over all traits; unestimated widths treated as zero
	double w=0.0;
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrTBParamRec *traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
		w += traitParams->width();
		}
	w /= traits();
	return w;//default
	}//end ClineMultiQuantTraitParamRec::width()



int ClineMultiQuantTraitParamRec::ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldCenter = center(trait), leftEnd, rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int centerSet = paramsShared[trait][0];
	do{
		double newCenter = ran.NormDev(oldCenter, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->center(newCenter);
		validChange = true;//traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			if(paramsOnOff[trait][2]) {
				validChange = traitParams->LJunction(leftEnd=low->xL_);}}
		if(validChange){
			if(paramsOnOff[trait][3]) {
				validChange = traitParams->RJunction(rightEnd=high->xR_);}}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->center(newCenter);
					validChange = true;//traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						if(paramsOnOff[tr][2]) {validChange = traitParams2->LJunction(low->xL_);}}
					if(validChange){
						if(paramsOnOff[tr][3]) {validChange = traitParams2->RJunction(high->xR_);}}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){//try again if any traits fail
						break;}
					}
				}//end for tr
			}
		SetCovMatrix();
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for center");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->center(oldCenter);
						if(paramsOnOff[tr][2]) {traitParams3->LJunction(leftEnd);}
						if(paramsOnOff[tr][3]) {traitParams3->RJunction(rightEnd);}
						}
					}//t
				SetCovMatrix();
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeCenter(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeD(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary D
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	double oldD = D_;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int DSet = paramsShared[0][6];//all D's either on or off together
	do{
		D_ = ran.NormDev(oldD, sigma);
		SetCovMatrix();
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for D");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				D_ = oldD;
				SetCovMatrix();}//restore
			return validChange=0;
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeD(Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, double weight){
	//typecasting trick, so don't use function from base class
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiQuantTraitParamRec* deviations = (ClineMultiQuantTraitParamRec*) paramDeviations->pointerToDerivedSelf;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	switch (param){
	 case 0:
	 	 return ChangeCenter(trait, ran, deviations->center(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeWidth(trait, ran, deviations->width(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 2:
	 	 return ChangeThetaL(trait, ran, deviations->thetaL(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 3:
	 	 return ChangeThetaR(trait, ran, deviations->thetaR(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 4:
	 	 return ChangeZL(trait, ran, deviations->zL(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 5:
	 	 return ChangeZR(trait, ran, deviations->zR(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 6:
	 	 return ChangeLeftScore(trait, ran, deviations->leftVal(0)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 7:
	 	 return ChangeRightScore(trait, ran, deviations->rightVal(0)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 8:
	 	 return ChangeD(ran, deviations->D()*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a multi-trait, 7-parameter cline model");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end ClineMultiQuantTraitParamRec::ChangeParam(int...)



int ClineMultiQuantTraitParamRec::ChangeLeftScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldLeft = leftVal(trait);
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int leftSet = paramsShared[trait][6];
	do{
		double newLeft = ran.NormDev(oldLeft, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->leftVal(newLeft,0);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][6]) && (leftSet == paramsShared[tr][6])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->leftVal(newLeft,0);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		SetCovMatrix();
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][6]) && (leftSet == paramsShared[tr][6])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->leftVal(oldLeft,0);
						}
					}//t
				SetCovMatrix();//restore
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeLeftScore(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeRightScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldRight = rightVal(trait);
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int rightSet = paramsShared[trait][7];
	do{
		double newRight = ran.NormDev(oldRight, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->rightVal(newRight,0);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][7]) && (rightSet == paramsShared[tr][7])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->rightVal(newRight,0);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(! validChange){
						break;}
					}
				}//end for tr
			SetCovMatrix();
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][7]) && (rightSet == paramsShared[tr][7])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->rightVal(oldRight,0);
						}
					}//t
				SetCovMatrix();//restore
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeRightScore(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeThetaL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldThetaL = thetaL(trait), leftEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int thetaSet = paramsShared[trait][2];
	do{
		double newThetaL = ran.NormDev(oldThetaL, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->thetaL(newThetaL);
		validChange = traitParams->LJunction(leftEnd=low->xL_);
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][2]) && (thetaSet == paramsShared[tr][2])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->thetaL(newThetaL);
					validChange = traitParams2->LJunction(low->xL_);
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			SetCovMatrix();
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][2]) && (thetaSet == paramsShared[tr][2])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->thetaL(oldThetaL);
						traitParams3->LJunction(leftEnd);
						}
					}//t
				SetCovMatrix();
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeThetaL(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiQuantTraitParamRec::ChangeThetaR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldThetaR = thetaR(trait), rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int thetaSet = paramsShared[trait][3];
	do{
		double newThetaR = ran.NormDev(oldThetaR, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->thetaR(newThetaR);
		validChange = traitParams->RJunction(rightEnd=high->xR_);
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][3]) && (thetaSet == paramsShared[tr][3])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->thetaR(newThetaR);
					validChange = traitParams2->RJunction(high->xR_);
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			SetCovMatrix();
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][3]) && (thetaSet == paramsShared[tr][3])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->thetaR(oldThetaR);
						traitParams3->RJunction(rightEnd);
						}
					}//t
				SetCovMatrix();
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeThetaR(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeWidth(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldWidth = width(trait), leftEnd, rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int widthSet = paramsShared[trait][1];
	do{
		double newWidth = ran.NormDev(oldWidth, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->width(newWidth);
		validChange= traitParams->MeetsParamCriteria(low, high);;
		if(validChange){
			if(paramsOnOff[trait][2]) {
				validChange = traitParams->LJunction(leftEnd=low->xL_);}}
		if(validChange){
			if(paramsOnOff[trait][3]) {
				validChange = traitParams->RJunction(rightEnd=high->xR_);}}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->width(newWidth);
					validChange= traitParams2->MeetsParamCriteria(low, high);;
					if(validChange){
						if(paramsOnOff[tr][2]) {
							validChange = traitParams2->LJunction(low->xL_);}}
					if(validChange){
						if(paramsOnOff[tr][3]) {
							validChange = traitParams2->RJunction(high->xR_);}}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			SetCovMatrix();
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for width");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->width(oldWidth);
						if(paramsOnOff[tr][2]) {traitParams3->LJunction(leftEnd);}
						if(paramsOnOff[tr][3]) {traitParams3->RJunction(rightEnd);}
						}
					}//t
				SetCovMatrix();
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeWidth(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeZL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldZL = zL(trait), leftEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int zSet = paramsShared[trait][4];
	do{
		double newZL = ran.NormDev(oldZL, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->zL(newZL);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			validChange = traitParams->LJunction(leftEnd=low->xL_);}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][4]) && (zSet == paramsShared[tr][4])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->zL(newZL);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						validChange = traitParams2->LJunction(leftEnd=low->xL_);}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			SetCovMatrix();
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for zL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][4]) && (zSet == paramsShared[tr][4])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->zL(oldZL);
						traitParams3->LJunction(leftEnd);
						}
					}//t
				SetCovMatrix();
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeZL(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiQuantTraitParamRec::ChangeZR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false, tries=0;
	double oldZR = zR(trait), rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int zSet = paramsShared[trait][5];
	do{
		double newZR = ran.NormDev(oldZR, sigma);
		ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[trait].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
		traitParams->zR(newZR);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			validChange = traitParams->RJunction(rightEnd=high->xR_);}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][5]) && (zSet == paramsShared[tr][5])){//change it too
					low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[tr].pointerToDerivedSelf;
					high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[tr].pointerToDerivedSelf;
					ClineShapeCWTlTrTBParamRec* traitParams2 = 
							(ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
					traitParams2->zR(newZR);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						validChange = traitParams2->RJunction(high->xR_);}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			SetCovMatrix();
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for zR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][5]) && (zSet == paramsShared[tr][5])){//change it back too
						ClineShapeCWTlTrTBParamRec* traitParams3 = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
						traitParams3->zR(oldZR);
						traitParams3->RJunction(rightEnd);//reset
						}
					}//t
				SetCovMatrix();
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiQuantTraitParamRec::ChangeZR(int, Ran3&, double, ParamRec*, ParamRec*)






double ClineMultiQuantTraitParamRec::D(void){
	return D_;
	}//end ClineMultiQuantTraitParamRec::D( )


void ClineMultiQuantTraitParamRec::D(double dd){
	D_ = dd;
	}//end ClineMultiQuantTraitParamRec::D( )



double ClineMultiQuantTraitParamRec::DispersalSigma(void){
	double aveSigma=0.0;
	for(int tr=0;tr<traits();++tr){
		aveSigma += DispersalSigma(tr);}
	aveSigma /= traits();
	return aveSigma;
	}//end ClineMultiQuantTraitParamRec::DispersalSigma()



double ClineMultiQuantTraitParamRec::DispersalSigma(int trait){
	ClineShapeCWTlTrTBParamRec *traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	double w = traitParams->width();
    if (D_ > 0.0){
		double sigma = sqrt((D_ * r_ * w * w) / (1.0 + r_));
		return sigma;}
	return 0.0;//default
	}//end ClineMultiQuantTraitParamRec::DispersalSigma()



long double ClineMultiQuantTraitParamRec::distanceBarrierL(void){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	double aveBarrier=0.0; double leftSideCount=0.0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][2]){//left side estimated
			aveBarrier += distanceBarrierL(tr);
			leftSideCount++;}
		}
	if(leftSideCount>0){aveBarrier /= leftSideCount;}
	return aveBarrier;
	}//end ClineMultiQuantTraitParamRec::distanceBarrierL()



long double ClineMultiQuantTraitParamRec::distanceBarrierL(int trait){//barrier to introgression in distance units
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]==0){return 0.0;}
    //derivatives found using Mathematica
	ClineShapeCWTlTrTBParamRec *traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	double w = traitParams->width();
    if (w ==0.0){
			return exp(100.0);}//huge and undefined
	double th = traitParams->thetaL();
	double XL = traitParams->xL();
	double c = traitParams->center();
	double ZL = traitParams->zL();
    long double derivFactor = 4.0 * sqrt(th) * (XL - (c + ZL)) / w;
    if (derivFactor <= -100.0){
			return exp(100.0);}//huge and undefined
    long double dPLdXatxL, beta=0.0;
    if (derivFactor <= 100.0){
			dPLdXatxL = exp(derivFactor) * sqrt(th) / w;}
    	else{
			dPLdXatxL = exp(100.0);}

    if(dPLdXatxL == 0.0){
			return exp(100.0);}
		else{
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPLdXatxL;}
	return beta;
	}//end ClineMultiQuantTraitParamRec::distanceBarrierL()
 


long double ClineMultiQuantTraitParamRec::distanceBarrierR(void){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	double aveBarrier=0.0; double rightSideCount=0.0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][3]){//right side estimated
			aveBarrier += distanceBarrierR(tr);
			rightSideCount++;}
		}
	if(rightSideCount>0){aveBarrier /= rightSideCount;}
	return aveBarrier;
	}//end ClineMultiQuantTraitParamRec::distanceBarrierR()



long double ClineMultiQuantTraitParamRec::distanceBarrierR(int trait){//barrier to introgression in distance units
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]==0){return 0.0;}
    //derivatives found using Mathematica
	ClineShapeCWTlTrTBParamRec *traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	double w = traitParams->width();
    if (w ==0.0){
			return exp(100.0);}//huge and undefined
	double th = traitParams->thetaR();
	double XR = traitParams->xR();
	double c = traitParams->center();
	double ZR = traitParams->zR();
    long double derivFactor = 4.0 * sqrt(th) * (XR - (c + ZR)) / w;
    if (derivFactor <= -100.0){
			return exp(100.0);}//huge and undefined
    long double dPRdXatxR, beta=0.0;
    if (derivFactor <= 100.0){
			dPRdXatxR = exp(derivFactor) * sqrt(th) / w;}
    	else{
			dPRdXatxR = exp(100.0);}

    if(dPRdXatxR == 0.0){
			return exp(100.0);}
		else{
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPRdXatxR;}
	return beta;
	}//end ClineMultiQuantTraitParamRec::distanceBarrierR()
 



void ClineMultiQuantTraitParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	lnLDiseq_=0.0;
	r_ = i.r_;
	paramsOnOff_ = i.paramsOnOff_;//copies pointer
	paramsShared_ = i.paramIdentities_;//copies pointer
	varyingParams_=0;//in case of reinitialization
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	traitParamSet_.Clear();//in case of reinitialization
	for(int tr=0;tr<i.traits();++tr){//account for on/off parameters here
		traitParamSet_.Append(NEW ClineShapeCWTlTrTBParamRec(i));
		ClineShapeCWTlTrTBParamRec *traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
		if(paramsOnOff[tr][2]==0 || paramsOnOff[tr][4]==0){//thetaL or zL off
			paramsOnOff[tr][2] = paramsOnOff[tr][4]=0;//both off if either is off
			traitParams->thetaL(1.0);
			traitParams->zL(0.0);
			traitParams->xL_=i.leftEnd_;
			}
		if(paramsOnOff[tr][3]==0 || paramsOnOff[tr][5]==0){//thetaR or zL off
			paramsOnOff[tr][3] = paramsOnOff[tr][5]=0;//both off if either is off
			traitParams->thetaR(1.0);
			traitParams->zR(0.0);
			traitParams->xR_=i.rightEnd_;
			}
		if(paramsOnOff[tr][6]==0){//left edge off
			traitParams->leftVal(0,0.0);}
		if(paramsOnOff[tr][7]==0){//right edge off
			traitParams->leftVal(0,1.0);}
		}
	D_ = i.D_;
	//count varying params
	for(int p=0;p<BaseParams();++p){//assumes paramsShared is sorted low to high
		int maxparamID =-1;
		for(int tr=0;tr<traits();++tr){
			if((paramsOnOff[tr][p]==1) && (maxparamID<paramsShared[tr][p])){
				varyingParams_++;
				maxparamID = paramsShared[tr][p];
				}//if
			}//tr
		}//p
	}//end ClineMultiQuantTraitParamRec::Initialize(ClineFitInitializer&)



double ClineMultiQuantTraitParamRec::leftVal(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return traitParams->leftVal(0);
	}//end ClineMultiQuantTraitParamRec::leftVal(int)



double ClineMultiQuantTraitParamRec::lnLDiseq(void){
		return lnLDiseq_;}

		
void ClineMultiQuantTraitParamRec::lnLDiseq(double lnLval){
		lnLDiseq_ = lnLval;}
		
		

int ClineMultiQuantTraitParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiQuantTraitParamRec* lower = (ClineMultiQuantTraitParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineMultiQuantTraitParamRec* upper = (ClineMultiQuantTraitParamRec*) upperLimits->pointerToDerivedSelf;
	if((D_ < lower->D_) || (D_ > upper->D_)){ 
		return false;}
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
		ParamRec& lowParams = lower->traitParamSet_[tr];//no need to cast these yet
		ParamRec& highParams = upper->traitParamSet_[tr];
		int paramsValid = traitParams->MeetsParamCriteria(&lowParams,&highParams);
		if(! paramsValid) return false;
		}//tr
	return true;
	}//end ClineMultiQuantTraitParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineMultiQuantTraitParamRec::MultiplyParam(int paramNum, double factor){
	//use to rescale a param deviation
	//typecasting trick, so don't use function from base class
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	if(param==8){//change D
		D_ *= factor;
		return;}
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
			traitParams->MultiplyParam(param,factor);
			}
		}//tr
	}//ClineMultiQuantTraitParamRec::MultiplyParam(int, double)



int ClineMultiQuantTraitParamRec::NumDerivedParams(void){
	if(derivedParams_ > (-1)){//default flag set by constructor
		return derivedParams_;}
	//otherwise calculate it here first
	derivedParams_=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	int D_on = false, leftTail=0, rightTail=0;
	if(paramsOnOff[0][8]==1) D_on = true;
	if(D_on){
		derivedParams_++;}//DisersalSigma() estimated
	for(int tr=0;tr<traits();++tr){
		int both=0;
		if(paramsOnOff[tr][2]==1){//thetaL && zL turned on
			derivedParams_+=3;//xL_(tr), P_xL(tr) and distanceBarrierL(tr)
			both++;leftTail++;
			if(D_on) derivedParams_+=2;//timeBarrierL(tr),s_perMarkerLocusL(tr)
			}
		if(paramsOnOff[tr][3]==1){//thetaR && zR turned on
			derivedParams_+=3;//xR_(tr), P_xR(tr) and distanceBarrierR(tr)
			both++;rightTail++;
			if(D_on) derivedParams_+=2;//timeBarrierR(tr), s_perMarkerLocusR(tr)
			}
		if(both>0){//either is turned on
			derivedParams_++;//deltaP
			}
		if(D_on){
			derivedParams_++;}//DisersalSigma(tr) estimated
		}//tr
	if(leftTail>0) derivedParams_++;//distanceBarrierL()
	if(rightTail>0) derivedParams_++;//distanceBarrierR()
	if(D_on){
		if(leftTail){derivedParams_++;}//timeBarrierL()
		if(rightTail){derivedParams_++;}//timeBarrierR()
		derivedParams_+=7;
		//WhybridOverWpure(),S(),s_star_hets(), s_star_ecotone(), 
		//NumSelectedLoci(),s_perReproIsolationLocus,
		//s_star_DomEcotone()
		}
	return derivedParams_;
	}//end ClineMultiQuantTraitParamRec::NumDerivedParams()



double ClineMultiQuantTraitParamRec::NumSelectedLoci(void){
	double s = s_star_vs_Hets();
    if (s == 0.0){
		return 0.0;}//no loci under selection if no selection
	double ww = WhybridOverWpure();
    if ((s >= 1.0) || (ww <= 0.0)){
		return 1.0e20;}//a really high number if fitness=0
    double nLocSel = 2.0 * ln(ww) / ln(1.0 - s);
    return nLocSel;
    }//end ClineMultiQuantTraitParamRec::NumSelectedLoci()





void ClineMultiQuantTraitParamRec::ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived){
	//returns values for trait && traitParam
	//if params covary among traits, this returns the first trait
	derived = false;//to start
	int count=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	for(int p=0;p<BaseParams();++p){//try base params first
		int maxSharedID=-1;
		for(int tr=0;tr<traits();++tr){
			if(paramsOnOff[tr][p]==1){//estimating this one
				if(maxSharedID<paramsShared[tr][p]){//this is a new param to estimate
					if(globalParamNum==count){
						trait=tr; traitParam=p;
						return;}
					++count;
					maxSharedID = paramsShared[tr][p];//assumes these are in ascending order
				}}
			}//tr
		}//p
	//if we're here, it must be a derived parameter
	int dp = globalParamNum - NumParams();
	derived = true;
	int D_on = false, leftTail=0, rightTail=0;
	if(paramsOnOff[0][6]==1) D_on = true;
	int dcount=0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][2]){
			leftTail++;
			if(dp==dcount){//xL estimated
				trait=tr; traitParam=0;return;}
			dcount++;}
		if(paramsOnOff[tr][2]){
			if(dp==dcount){//p[xL] estimated
				trait=tr; traitParam=1;return;}
			dcount++;}
		if(paramsOnOff[tr][2]){
			if(dp==dcount){//distanceBarrierL estimated
				trait=tr; traitParam=2;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			rightTail++;
			if(dp==dcount){//xR estimated
				trait=tr; traitParam=3;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			if(dp==dcount){//p[xR] estimated
				trait=tr; traitParam=4;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			if(dp==dcount){//distanceBarrierR estimated
				trait=tr; traitParam=5;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][2]){
			if(dp==dcount){//timeBarrierL estimated
				trait=tr; traitParam=6;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][2]){
			if(dp==dcount){//s_perMarkerLocusL estimated
				trait=tr; traitParam=7;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][3]){
			if(dp==dcount){//timeBarrierR estimated
				trait=tr; traitParam=8;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][3]){
			if(dp==dcount){//s_perMarkerLocusR estimated
				trait=tr; traitParam=9;return;}
			dcount++;}
		if(paramsOnOff[tr][2] || paramsOnOff[tr][3]){
			if(dp==dcount){//deltaP estimated
				trait=tr; traitParam=10;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][3]){
			if(dp==dcount){//dispersalSigma(tr) estimated
				trait=tr; traitParam=11;return;}
			dcount++;}
		}//tr
	//getting here implies that the parameter must be shared among traits
	int dpshared = dp - dcount;	
	dcount=0;
	trait=-1;//flag for this situation
	if(D_on){
		if(dpshared==dcount){//ave. DispersalSigma estimated
			traitParam=0;return;}
		dcount++;}
	if(leftTail){
		if(dpshared==dcount){//ave. distanceBarrierL estimated
			traitParam=1;return;}
		dcount++;}
	if(D_on && leftTail){
		if(dpshared==dcount){//ave. timeBarrierL estimated
			traitParam=2;return;}
		dcount++;}
	if(rightTail){
		if(dpshared==dcount){//ave. distanceBarrierR estimated
			traitParam=3;return;}
		dcount++;}
	if(D_on && rightTail){
		if(dpshared==dcount){//ave. timeBarrierR estimated
			traitParam=4;return;}
		dcount++;}
	if(D_on){
		if(dpshared==dcount){//S estimated
			traitParam=5;return;}
		dcount++;
		if(dpshared==dcount){//Wh/Wp estimated
			traitParam=6;return;}
		dcount++;
		if(dpshared==dcount){//s*[hets] estimated
			traitParam=7;return;}
		dcount++;
		if(dpshared==dcount){//s*[ecotone] estimated
			traitParam=8;return;}
		dcount++;
		if(dpshared==dcount){//s[reproIsolationLocus] estimated
			traitParam=9;return;}
		dcount++;
		if(dpshared==dcount){//n[selLoci] estimated
			traitParam=10;return;}
		dcount++;
		if(dpshared==dcount){//s*[domEcotone] estimated
			traitParam=11;return;}
		}//D_on
	//exhausted possibilities, so return an error message
	derived=false;
	aString err("Can\'t find param");err.cat(globalParamNum+1);
	aError(err);
	}//end ClineMultiQuantTraitParamRec::ParamCoords(int, int&, int&, int&)




aString* ClineMultiQuantTraitParamRec::ParamName(int paramNum){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	int trait=0, param=0, origparam = paramNum, allShared=true, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	aString trStr,  *newName = NEW aString();
	if(! derived){//it's a primary parameter
			if(traits()>1){
				ObjList<aList<int> >& paramsShared = *paramsShared_;
				int shareState = paramsShared[trait][param];
				trStr="[";trStr.cat(trait+1);
				for(int tr=0;tr<traits();++tr){//append names of any shared traits
					if(tr != trait && paramsOnOff[tr][param]){
						if(paramsShared[tr][param]== shareState){
								trStr.cat(';'); trStr.cat(tr+1);}
							else{allShared=false;}
							}}
				trStr.cat("]");}
			switch (param){
			 case 0:
				newName->cat("center"); break;
			 case 1:
				newName->cat("width"); break;
			 case 2:
				newName->cat("thetaL"); break;
			 case 3:
				newName->cat("thetaR"); break;
			 case 4:
				newName->cat("zL"); break;
			 case 5:
				newName->cat("zR"); break;
			 case 6:
				newName->cat("pL"); break;
			 case 7:
				newName->cat("pR"); break;
			 case 8:
				newName->cat("D"); return newName; break;
			 default:
				aString errstr("bug: can't return name of param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");errstr.cat(trait+1);}
				aError(errstr);
			 	return newName;//null
			 	}	
			if(traits()>1 && !allShared){newName->cat(trStr);}
			}//if paramNum
		else{//it's a derived parameter
			if(trait>-1){//it is specific to a trait
					trStr.cat('[');trStr.cat(trait+1);trStr.cat(']');
					switch (param){
						 case 0:
							newName->cat("xL"); break;
						 case 1:
							newName->cat("p[xL]"); break;
						 case 2:
							newName->cat("distBarrierL"); break;
						 case 3:
							newName->cat("xR"); break;
						 case 4:
							newName->cat("p[xR]"); break;
						 case 5:
							newName->cat("distBarrierR"); break;
						 case 6:
							newName->cat("timeBarrierL"); break;
						 case 7:
							newName->cat("s_perMarkerLocusL"); break;
						 case 8:
							newName->cat("timeBarrierR"); break;
						 case 9:
							newName->cat("s_perMarkerLocusR"); break;
						 case 10:
							newName->cat("DeltaP"); break;
						 case 11:
							newName->cat("DispersalSigma"); break;
						 default:
						 	aString errstr("bug: can't return name of param #");
							errstr.cat(param+1);
							if(traits()>1){
								errstr.cat(" for trait #");errstr.cat(trait+1);}
							aError(errstr);
						 	return newName;
						 	break;}
					if(traits()>1){newName->cat(trStr);}
					return newName;
					}
				else{//parameter is commonly estimated
					switch (param){
						 case 0:
							newName->cat("DispersalSigma"); break;
						 case 1:
							newName->cat("distBarrierL"); break;
						 case 2:
							newName->cat("timeBarrierL"); break;
						 case 3:
							newName->cat("distBarrierR"); break;
						 case 4:
							newName->cat("timeBarrierR"); break;
						 case 5:
							newName->cat("S"); break;
						 case 6:
							newName->cat("Wh/Wp"); break;
						 case 7:
							newName->cat("s*[hets]"); break;
						 case 8:
							newName->cat("s*[ecotone]"); break;
						 case 9:
							newName->cat("s[reproIsolationLocus]"); break;
						 case 10:
							newName->cat("n[selectedLoci]"); break;
						 case 11:
							newName->cat("s*[domEcotone]"); break;
						 default:
						 	aString errstr("bug: can't return name of co-estimated derived param #");errstr.cat(param+1);
							aError(errstr);
						 	break;}
					}//else..
			}//derived
	return newName;			
	}//end ClineMultiQuantTraitParamRec::ParamName(int )



void ClineMultiQuantTraitParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::ParamString(dataline, sepchar);
	double val=0.0;
	//each param will be printed, whether or not it is co-estimated with another
	int D_on = paramsOnOff[0][8];
	if(D_on){
		val = lnLDiseq();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = D();
		dataline.cat(sepchar);dataline.cat(val,6,9);}
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
		if(paramsOnOff[tr][0]==1){//center
			val = traitParams->center();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][1]==1){//width
			val = traitParams->width();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][2]==1){//thetaL
			val = traitParams->thetaL();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][3]==1){//thetaR
			val = traitParams->thetaR();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][4]==1){//zL
			val = traitParams->zL();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][5]==1){//zR
			val = traitParams->zR();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][6]==1){//leftVal
			val = traitParams->leftVal(0);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][7]==1){//rightVal
			val = traitParams->rightVal(0);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][2]==1){//xL
			val = traitParams->xL();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][3]==1){//xR
			val = traitParams->xR();
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][2]==1){//left side estimated
			val = traitParams->B_L(tr);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][3]==1){//right side estimated
			val = traitParams->B_R(tr);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either side estimated
			val = traitParams->DeltaP(tr);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			}
		if(D_on){
			val = DispersalSigma(tr);
			dataline.cat(sepchar);dataline.cat(val,6,9);//estimated for this trait
			}
		if(paramsOnOff[tr][2]==1 ){//left side estimated
			val = distanceBarrierL(tr);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			if(D_on){
				val = timeBarrierL(tr);
				dataline.cat(sepchar);dataline.cat(val,6,9);
				val = s_perMarkerLocusL(tr);
				dataline.cat(sepchar);dataline.cat(val,6,9);
				}
			}//left side
		if(paramsOnOff[tr][3]==1 ){//right side estimated
			val = distanceBarrierR(tr);
			dataline.cat(sepchar);dataline.cat(val,6,9);
			if(D_on){
				val = timeBarrierR(tr);
				dataline.cat(sepchar);dataline.cat(val,6,9);
				val = s_perMarkerLocusR(tr);
				dataline.cat(sepchar);dataline.cat(val,6,9);
				}
			}//right side
		}//tr
	//parameters that average over traits
	if(D_on ){
		val = DispersalSigma();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = distanceBarrierL();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = timeBarrierL();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = distanceBarrierR();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = timeBarrierR();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = S();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = WhybridOverWpure();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = s_star_vs_Hets();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = s_star_Ecotone();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = s_perReproIsolationLocus();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = NumSelectedLoci();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		val = s_star_DomEcotone();
		dataline.cat(sepchar);dataline.cat(val,6,9);
		}//D_on

	}//end ClineMultiQuantTraitParamRec::ParamString(aString&, char)



double ClineMultiQuantTraitParamRec::ParamValue(int paramNum){
	//typecasting trick, so don't use function from base class
	//returns value of parameter; first varying params, then derived params
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	int trait=0, param=0, origparam = paramNum, allShared=false, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	if(! derived){//it's a primary parameter
			ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
			switch (param){
			 case 0:
				return traitParams->center(); break;
			 case 1:
				return traitParams->width(); break;
			 case 2:
				return traitParams->thetaL(); break;
			 case 3:
				return traitParams->thetaR(); break;
			 case 4:
				return traitParams->zL(); break;
			 case 5:
				return traitParams->zR(); break;
			 case 6:
				return traitParams->leftVal(); break;
			 case 7:
				return traitParams->rightVal(); break;
			 case 8:
				return D(); break;
			 default:
				aString errstr("bug: can't return value of param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");errstr.cat(trait+1);}
				aError(errstr);
			 	return 0.0; break;
			 	}	
			}//if paramNum
		else{//it's a derived parameter
			if(trait>-1){//it is specific to a trait
					ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
					switch (param){
						 case 0:
							return traitParams->xL(); break;
						 case 1:
							return traitParams->B_L(); break;
						 case 2:
							return distanceBarrierL(trait); break;
						 case 3:
							return traitParams->xR(); break;
						 case 4:
							return traitParams->B_R(); break;
						 case 5:
							return distanceBarrierR(trait); break;
						 case 6:
							return timeBarrierL(trait); break;
						 case 7:
							return s_perMarkerLocusL(trait); break;
						 case 8:
							return timeBarrierR(trait); break;
						 case 9:
							return s_perMarkerLocusR(trait); break;
						 case 10:
							return traitParams->DeltaP(); break;
						 case 11:
							return DispersalSigma(trait); break;
						 default:
						 	aString errstr("bug: can't return value of derived param #");
							errstr.cat(param+1);
							if(traits()>1){
								errstr.cat(" for trait #");errstr.cat(trait+1);}
							aError(errstr);
						 	break;}
					}
				else{//parameter is commonly estimated
					switch (param){
						 case 0:
							return DispersalSigma(); break;
						 case 1:
							return distanceBarrierL(); break;
						 case 2:
							return timeBarrierL(); break;
						 case 3:
							return distanceBarrierR(); break;
						 case 4:
							return timeBarrierR(); break;
						 case 5:
							return S(); break;
						 case 6:
							return WhybridOverWpure(); break;
						 case 7:
							return s_star_vs_Hets(); break;
						 case 8:
							return s_star_Ecotone(); break;
						 case 9:
							return s_perReproIsolationLocus(); break;
						 case 10:
							return NumSelectedLoci(); break;
						 case 11:
							return s_star_DomEcotone(); break;
						 default:
						 	aString errstr("bug: can't return value of co-estimated derived param #");errstr.cat(param+1);
							aError(errstr);
							return 0.0;
						 	break;}
					}//else..
			}//derived
	return 0.0;
	}//end ClineMultiQuantTraitParamRec::ParamValue(int )




void ClineMultiQuantTraitParamRec::Print(ostream& outFile, long rep, aString& message){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::Print(outFile, rep, message);
	//for now, each param will be printed, whether or not it is co-estimated with another
	aString outStr;
	for(int tr=0;tr<traits();++tr){
		outStr.clear();
		ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
		if(paramsOnOff[tr][0]==1){//center
			outStr.cat("center");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->center();
			outStr.cat(val,6,4);
			outStr.separator(5);
			}
		if(paramsOnOff[tr][1]==1){//width
			outStr.cat("width");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->width();
			outStr.cat(val,6,4);
			}
		outFile<<outStr<<endl;
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//thetaL & zL toggle on/off together
			outStr="thetaL";
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->thetaL();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("zL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->zL();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("xL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->xL();
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		outStr.clear();
		if(paramsOnOff[tr][3]==1){//thetaR & zR toggle on/off together
			outStr.cat("thetaR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->thetaR();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("zR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->zR();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("xR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->xR();
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		outStr.clear();
		if(paramsOnOff[tr][6]==1){//left frequency estimated
			outStr.cat("P_left");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->leftVal(0);
			outStr.cat(val,6,4);
			}
		if(paramsOnOff[tr][7]==1){//right frequency estimated
			outStr.separator(2);
			outStr.cat("P_right");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->rightVal(0);
			outStr.cat(val,6,4);
			}
		if(outStr.len()>0){
			outFile<<outStr<<endl;}
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//left tail estimated
			outStr.cat("P_xL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->B_L(0);
			outStr.cat(val,6,4);
			}
		if(paramsOnOff[tr][3]==1){//right tail estimated
			outStr.separator(2);
			outStr.cat("P_xR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->B_R(0);
			outStr.cat(val,6,4);
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either tail estimated
			outStr.separator(2);
			outStr.cat("deltaP");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->DeltaP(0);
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		}//tr
	outStr="lnLdiseq: ";outStr.cat(lnLDiseq(),6,9);
	outStr.separator(4);outStr.cat("D: ");outStr.cat(D(),6,9);
	outFile<<outStr<<endl;
	outStr="dispersal sigma: ";outStr.cat(DispersalSigma(),6,4);
	outStr.separator(2);outStr.cat("S: ");outStr.cat(S(),6,9);
	outStr.separator(2);outStr.cat("Wh/Wp: ");outStr.cat(WhybridOverWpure(),6,9);
	outFile<<outStr<<endl;
	outStr="s*[hets]: ";outStr.cat(s_star_vs_Hets(),6,4);
	outStr.separator(2);outStr.cat("s*[ecotone]: ");outStr.cat(s_star_Ecotone(),6,4);
	outFile<<outStr<<endl;
	outStr="s*[domEcotone]: ";outStr.cat(s_star_DomEcotone(),6,4);
	outStr.separator(2);outStr.cat("s[repro_Iso_Locus]: ");outStr.cat(s_perReproIsolationLocus(),6,4);
	outFile<<outStr<<endl;
	outStr="n[selected loci]: ";outStr.cat(NumSelectedLoci(),6,4);
	outStr.separator(2);outStr.cat("s[repro_Iso_Locus]: ");outStr.cat(s_perReproIsolationLocus(),6,4);
	outFile<<outStr<<endl;
	}//ClineMultiQuantTraitParamRec::Print



double ClineMultiQuantTraitParamRec::r(void){
	return r_;}


void ClineMultiQuantTraitParamRec::r(double rr){
	r_ = rr;}
	

double ClineMultiQuantTraitParamRec::rightVal(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return traitParams->rightVal(0);
	}//end ClineMultiQuantTraitParamRec::rightVal()


double ClineMultiQuantTraitParamRec::S(void){
    if (WhybridOverWpure() == 0.0){
		return 200.0;}//a high number
    double bigS = -2.0 * ln(WhybridOverWpure());
    return bigS;
    }//end ClineMultiQuantTraitParamRec::S()



void ClineMultiQuantTraitParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter, and all those that covary with it
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	if(param==8){//D only
		D_ = newVal;
		return;}//else...
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[tr].pointerToDerivedSelf;
			traitParams->SetParamValue(param,newVal);
			}
		}//tr
	}//end ClineMultiQuantTraitParamRec::SetParamValue(int,long double)



double ClineMultiQuantTraitParamRec::s_perMarkerLocusL(int trait){
	//direct selection on marker loci in left tail of cline
	//assuming genetic background incompatibilities
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]==0){return 0.0;}
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	if(traitParams->width()==0.0){return 0.0;}//undefined
    double s = traitParams->thetaL() * s_star_vs_Hets();
    return s;
    }//end ClineMultiQuantTraitParamRec::s_perMarkerLocusL(int)


double ClineMultiQuantTraitParamRec::s_perMarkerLocusR(int trait){
	//direct selection on marker loci in right tail of cline
	//assuming genetic background incompatibilities
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]==0){return 0.0;}
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	if(traitParams->width()==0.0){return 0.0;}//undefined
    double s = traitParams->thetaR() * s_star_vs_Hets();
    return s;
    }//end ClineMultiQuantTraitParamRec::s_perMarkerLocusR(int)


double ClineMultiQuantTraitParamRec::s_perReproIsolationLocus(void){
	//average selection on a locus contributing to reproductive isolation
	double n = NumSelectedLoci();
	 if (n == 0.0){
			return 0.0;}
    return S() / n;
    }//end ClineMultiQuantTraitParamRec::s_perReproIsolationLocus(void)




double ClineMultiQuantTraitParamRec::s_star_vs_Hets(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming genetic background incompatibilities
	double w = aveWidth();
	if(w==0.0){return 0.0;}//undefined
    double s = sqr(2.0 * DispersalSigma() / w);
    return s;
    }//end ClineMultiQuantTraitParamRec::s_star_vs_Hets()


double ClineMultiQuantTraitParamRec::s_star_Ecotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone
	double w = aveWidth();
	if(w==0.0){return 0.0;}//undefined
    double s = sqr(1.732 * DispersalSigma() / w);
    return s;
    }//end ClineMultiQuantTraitParamRec::s_star_Ecotone()


double ClineMultiQuantTraitParamRec::s_star_DomEcotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone on a dominant marker
	double w = aveWidth();
	if(w==0.0){return 0.0;}//undefined
    double s = sqr(1.782 * DispersalSigma() / w);
    return s;
    }//end ClineMultiQuantTraitParamRec::s_star_DomEcotone()




long double ClineMultiQuantTraitParamRec::timeBarrierL(void){
	long double d = DispersalSigma();
    if (d == 0.0){
		return 1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL();
    if (b / d > 1.0e40){
		return 1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);
    return timeL;
}//end ClineMultiQuantTraitParamRec::timeBarrierL()


long double ClineMultiQuantTraitParamRec::timeBarrierL(int trait){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]==0){return 0.0;}
	long double d = DispersalSigma(trait);
    if (d == 0.0){
		return 1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL(trait);
    if (b / d > 1.0e40){
		return 1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);
    return timeL;
}//end ClineMultiQuantTraitParamRec::timeBarrierL(int)


long double ClineMultiQuantTraitParamRec::timeBarrierR(void){
	long double d = DispersalSigma();
    if (d == 0.0){
		return 1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR();
    if (b / d > 1.0e40){
		return 1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);
    return timeR;
}//end ClineMultiQuantTraitParamRec::timeBarrierR()


long double ClineMultiQuantTraitParamRec::timeBarrierR(int trait){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]==0){return 0.0;}
	long double d = DispersalSigma(trait);
    if (d == 0.0){
		return 1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR(trait);
    if (b / d > 1.0e40){
		return 1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);
    return timeR;
}//end ClineMultiQuantTraitParamRec::timeBarrierR(int)


void ClineMultiQuantTraitParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the names of values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::TitleString(dataline, sepchar);
	//each param will be printed, whether or not it is co-estimated with another
	int D_on = paramsOnOff[0][8];
	if(D_on){
		dataline.cat(sepchar);
		dataline.cat("lnLDiseq");
		dataline.cat(sepchar);
		dataline.cat("D");}
	for(int tr=0;tr<traits();++tr){
		aString trStr('[');trStr.cat(tr+1);trStr.cat(']');
		if(paramsOnOff[tr][0]==1){//center
			dataline.cat(sepchar);dataline.cat("center");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][1]==1){//width
			dataline.cat(sepchar);dataline.cat("width");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1){//thetaL
			dataline.cat(sepchar);dataline.cat("thetaL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][3]==1){//thetaR
			dataline.cat(sepchar);dataline.cat("thetaR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][4]==1){//zL
			dataline.cat(sepchar);dataline.cat("zL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][5]==1){//zR
			dataline.cat(sepchar);dataline.cat("zR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][6]==1){//zR
			dataline.cat(sepchar);dataline.cat("P_left");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][7]==1){//zR
			dataline.cat(sepchar);dataline.cat("P_right");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1){//xL
			dataline.cat(sepchar);dataline.cat("xL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][3]==1){//xR
			dataline.cat(sepchar);dataline.cat("xR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1){//left side estimated
			dataline.cat(sepchar);dataline.cat("P_at_xL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][3]==1){//right side estimated
			dataline.cat(sepchar);dataline.cat("P_at_xR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either side estimated
			dataline.cat(sepchar);dataline.cat("deltaP");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(D_on){
			dataline.cat(sepchar);dataline.cat("dispersalSigma");//estimated for this trait
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1 ){//left side estimated
			dataline.cat(sepchar);dataline.cat("distBarrierL");
			if(traits()>1){dataline.cat(trStr);}
			if(D_on){
				dataline.cat(sepchar);dataline.cat("timeBarrierL");
				if(traits()>1){dataline.cat(trStr);}
				dataline.cat(sepchar);dataline.cat("s_perMarkerLocusL");
				if(traits()>1){dataline.cat(trStr);}
				}
			}//left side
		if(paramsOnOff[tr][3]==1 ){//right side estimated
			dataline.cat(sepchar);dataline.cat("distBarrierR");
			if(traits()>1){dataline.cat(trStr);}
			if(D_on){
				dataline.cat(sepchar);dataline.cat("timeBarrierR");
				if(traits()>1){dataline.cat(trStr);}
				dataline.cat(sepchar);dataline.cat("s_perMarkerLocusR");
				if(traits()>1){dataline.cat(trStr);}
				}
			}//right side
		}//tr
	//parameters that average over traits
	if(D_on ){
		dataline.cat(sepchar);dataline.cat("dispersalSigma");
		dataline.cat(sepchar);dataline.cat("distBarrierL");
		dataline.cat(sepchar);dataline.cat("timeBarrierL");
		dataline.cat(sepchar);dataline.cat("distBarrierR");
		dataline.cat(sepchar);dataline.cat("timeBarrierR");
		dataline.cat(sepchar);dataline.cat("S");
		dataline.cat(sepchar);dataline.cat("Wh/Wp");
		dataline.cat(sepchar);dataline.cat("s*[hets]");
		dataline.cat(sepchar);dataline.cat("s*[ecotone]");
		dataline.cat(sepchar);dataline.cat("s[reproIsolationLocus]");
		dataline.cat(sepchar);dataline.cat("n[selectedLoci]");
		dataline.cat(sepchar);dataline.cat("s*[domEcotone]");
		}//D_on

	}//end ClineMultiQuantTraitParamRec::TitleString(aString&, char)



double ClineMultiQuantTraitParamRec::WhybridOverWpure(void){
	double w = aveWidth();
	if(w==0){return 0.0;}//undefined
	long double beta = (distanceBarrierL() + distanceBarrierR()) / 2.0;
    if ((beta / w > 3000.0) || (beta / w == 0.0)){
		return 0.0;}
    double wratio = exp(- r_ * (ln(beta / w)));
    return wratio;
	}//end ClineMultiQuantTraitParamRec::WhybridOverWpure()





void ClineMultiQuantTraitParamRec::operator=(ParamRec& p){
	ParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineMultiQuantTraitParamRec
	ClineMultiQuantTraitParamRec* pp = (ClineMultiQuantTraitParamRec*) p.pointerToDerivedSelf;
	paramsOnOff_ = pp->paramsOnOff_;
	paramsShared_ = pp->paramsShared_;
	varyingParams_ = pp->varyingParams_;
	derivedParams_ = pp->derivedParams_;
	D_ = pp->D_;
	lnLDiseq_ = pp->lnLDiseq_;	
	r_ = pp->r_;
	traitParamSet_.Clear();
	for(int tr=0;tr<pp->traits();++tr){
		ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) pp->traitParamSet_[tr].pointerToDerivedSelf;
		traitParamSet_.Append(NEW ClineShapeCWTlTrTBParamRec(*traitParams));
		}//tr
	}//end  ClineMultiQuantTraitParamRec::operator=(ParamRec&)





int	operator==(ClineMultiQuantTraitParamRec& p, ClineMultiQuantTraitParamRec& q){
	if(p.traits()!=q.traits()) return 0;
	if(p.D_ != q.D_) return 0;
	if(p.r_ != q.r_) return 0;
	for(int tr=0;tr<p.traits();++tr){
		ClineShapeCWTlTrTBParamRec* traitParams1 = 
			(ClineShapeCWTlTrTBParamRec*) p.traitParamSet_[tr].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* traitParams2 = 
			(ClineShapeCWTlTrTBParamRec*) q.traitParamSet_[tr].pointerToDerivedSelf;
		if(*traitParams1 != *traitParams2){return false;}
		}
	if(p.paramsOnOff_ != q.paramsOnOff_) return false;//compares pointers
	if(p.paramsShared_ != q.paramsShared_) return false;//compares pointers
	return true;
	}//end operator==(ClineMultiQuantTraitParamRec&, ClineMultiQuantTraitParamRec&)


int	operator!=(ClineMultiQuantTraitParamRec& p, ClineMultiQuantTraitParamRec& q){
	return !(p==q);
	}//end operator!=(ClineMultiQuantTraitParamRec&, ClineMultiQuantTraitParamRec&)



#endif


