/******************************************************
*
*	ClineMultiQuantTraitParamRec.h
*
*	Parameters for a multi-trait, 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL, zR, and left and right quantitative trait values.  The middle 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.  Disequilibrium
*	is also estimated and derived parameters calculated.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineMultiQuantTraitParamRech
#define ClineMultiQuantTraitParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineFitInitializer.h"
#include "ClineGeneticData.h"
#include "ClineMultiCWTlTrTBParamRec.h"
#include "ClineShapeCWTlTrTBParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "MatrixList.h"
#include "MemoryManager.h"

class ClineMultiQuantTraitParamRec;
class ClineMultiQuantTraitParamRec: public ClineMultiCWTlTrTBParamRec{
  public:
  	//uses variables & data held in ClineMultiCWTlTrTBParamRec
  	double lnLDiseq_;
  	double D_;
  	double r_;//harmonic mean linkage of selected to random traits
  	MatrixList cov_;//variance-covariance matrices, one for each cline region; index [region][trait][trait]
  	MatrixList invCov_;//inverses of variance-covariance matrices; index [region][trait][trait]
  	aList<double> detCov_;//determinants of variance-covariance matrices; index [region]
  	aList<int> invCovMatrixFlags_;//flags; index [region]
  	aList<int> detCovMatrixFlags_;//flags; index [region]
  public:
  	ClineMultiQuantTraitParamRec(void):ClineMultiCWTlTrTBParamRec(),lnLDiseq_(0.0),D_(0.0),r_(0.5){
  				SetPointerToDerivedSelf();//empty parameter set
  				}
  	ClineMultiQuantTraitParamRec(ClineFitInitializer& i):
  					ClineMultiCWTlTrTBParamRec(),lnLDiseq_(0.0),D_(0.0),r_(0.5){//initialize empty, then fill here
  				SetPointerToDerivedSelf();
  				Initialize(i);
  				}
  	ClineMultiQuantTraitParamRec(ClineMultiQuantTraitParamRec& p):
  					ClineMultiCWTlTrTBParamRec(){//initialize empty, then fill here
  				lnL_ = p.lnL_;
  				lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;
  				paramsOnOff_ = p.paramsOnOff_;
  				paramsShared_ = p.paramsShared_;
  				varyingParams_ = p.varyingParams_;
  				derivedParams_ = p.derivedParams_;
  				lnLDiseq_ = p.lnLDiseq_;
  				D_ = p.D_;
  				r_ = p.r_;
  				cov_=p.cov_;
  				invCov_=p.invCov_;
  				detCov_=p.detCov_;
  				invCovMatrixFlags_=p.invCovMatrixFlags_;
  				detCovMatrixFlags_=p.detCovMatrixFlags_;
  				SetPointerToDerivedSelf();
  				for(int tr=0;tr<p.traits();++tr){
  					ClineShapeCWTlTrTBParamRec* traitparams = (ClineShapeCWTlTrTBParamRec*) p.traitParamSet_[tr].pointerToDerivedSelf;
  					traitParamSet_.Append(NEW ClineShapeCWTlTrTBParamRec(*traitparams));}
				}
				
  	virtual ~ClineMultiQuantTraitParamRec(void){
		  		cov_.Clear();
		  		invCov_.Clear();
		  		detCov_.Clear();
		  		invCovMatrixFlags_.Clear();
		  		detCovMatrixFlags_.Clear();
		  		}//rest handled by base class
		  			

  	virtual ParamRec* 					Duplicate(void);
  	virtual ClineMultiQuantTraitParamRec* 	_Duplicate(void);
	virtual void 						SetPointerToDerivedSelf(void);//always override in derived
  	


	virtual double 		aveWidth(void);//average width over all traits
  	virtual double 		D(void);//returns D_
  	virtual void 		D(double dd);//sets D_
	virtual double 		leftVal(int trait);//returns value
//			void 		leftVal(int trait, double lft);//sets value
			double 		lnLDiseq(void);//returns lnLDiseq_
			void 		lnLDiseq(double lnLval);//sets lnLDiseq_
	virtual double 		r(void);//returns r_
	virtual void 		r(double rr);//sets r_
	virtual double 		rightVal(int trait);//returns value
//			void 		rightVal(int trait, double rt);//sets value
	virtual long double distanceBarrierL(void);//barrier to introgression from left to right in distance units
	virtual long double distanceBarrierR(void);//barrier to introgression from right to left in distance units
	virtual long double distanceBarrierL(int trait);//barrier to introgression from left to right in distance units
	virtual long double distanceBarrierR(int trait);//barrier to introgression from right to left in distance units

	virtual int 		BaseParams(void){return 9;}
//	virtual double 		Cline_at_X(double x, int trait);
  	
	virtual int 		ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeD(Ran3& ran,double sigma,
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeLeftScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, double weight=1.0);
	virtual int 		ChangeRightScore(int  trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeWidth(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZL(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZR(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual double 		DispersalSigma(void);//average over traits
	virtual double 		DispersalSigma(int trait);
//	virtual double 		ExpectedValue(aList<double>& coords);
	virtual void 		Initialize(ClineFitInitializer& i);
//	virtual int 		LJunction(int trait, double leftEnd);//finds point where left and center cline sections meet
//	virtual int 		RJunction(int trait, double rightEnd);//finds point where right and center cline sections meet
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, double factor);//use to rescale a param deviation
//	virtual int 		NumParams(void);//number of varying parameters; correctly handled in base class
	virtual int 		NumDerivedParams(void);//number of params that depend on varying params
	virtual double 		NumSelectedLoci(void);//assumes many loci and selection via genetic background
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived);
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual double 		ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual double 		S(void);//effective selection against hybrids in center as if hybrids were single-locus heterozygotes
	virtual void 		SetCovMatrix(void);//sets covariances based on D and trait expected values
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual double 		s_perMarkerLocusL(int trait);//ave. direct selection on marker loci in left tail of cline
	virtual double 		s_perMarkerLocusR(int trait);//ave. direct selection on marker loci in right tail of cline
	virtual double 		s_perReproIsolationLocus(void);//average selection on a locus contributing to reproductive isolation
	
						//indirect & direct selection on marker loci in middle of cline
	virtual double 		s_star_vs_Hets(void);//assuming genetic background incompatibilities
	virtual double 		s_star_Ecotone(void);//assuming selection across an ecotone
	virtual double 		s_star_DomEcotone(void);//assuming selection across an ecotone on a dominant marker

	virtual long double timeBarrierL(void);
	virtual long double timeBarrierL(int trait);
	virtual long double timeBarrierR(void);
	virtual long double timeBarrierR(int trait);
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
//	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters
//	virtual int 		traits(void);//number of traits -- handled by base class
	virtual	double 		WhybridOverWpure(void);//ratio in cline center

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineMultiQuantTraitParamRec& p, ClineMultiQuantTraitParamRec& q);
	friend int	operator!=(ClineMultiQuantTraitParamRec& p, ClineMultiQuantTraitParamRec& q);

	};//end class ClineMultiQuantTraitParamRec



#endif