/******************************************************
*
*	ClineShapeCWTlTrDParamRec.cpp
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.  Disequilibrium
*	is also estimated and derived parameters calculated.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrDParamReccpp
#define ClineShapeCWTlTrDParamReccpp = 1

#include "ClineShapeCWTlTrDParamRec.h"

	
ClineShapeCWTlTrDParamRec::ClineShapeCWTlTrDParamRec(void):
				r_((long double)0.5){
  	SetPointerToDerivedSelf();
  	}


ClineShapeCWTlTrDParamRec::ClineShapeCWTlTrDParamRec(long double c, long double w,
													 long double tL, long double tR,
													long double zL, long double zR,
													long double r, long double D):
					r_(r){
  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;	
  	c_=c; w_=w;
  	thetaL_=tL; thetaR_=tR;
  	zL_=zL; zR_=zR;
  	D_=D; lnLDiseq_=0.0;
  	}


ClineShapeCWTlTrDParamRec::ClineShapeCWTlTrDParamRec(long double c, long double w,
													 long double tL, long double tR,
													 long double zL, long double zR,
													 long double r, long double D,
													double leftEnd, double rightEnd):
					r_(r){
  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;	
  	c_=c; w_=w;
  	thetaL_=tL; thetaR_=tR;
  	zL_=zL; zR_=zR;
  	D_=D; lnLDiseq_=0.0;
  	LJunction(leftEnd);
  	RJunction(rightEnd);
  	}


ClineShapeCWTlTrDParamRec::ClineShapeCWTlTrDParamRec(ClineFitInitializer& i):
					r_(i.r_){
  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;	
  	c_=i.c_; w_=i.w_;
  	thetaL_=i.thetaL_; thetaR_=i.thetaR_;
  	zL_=i.zL_; zR_=i.zR_;
  	D_=i.D_; lnLDiseq_=0.0;
  	LJunction(i.leftEnd_);
  	RJunction(i.rightEnd_);
  	}


ClineShapeCWTlTrDParamRec::ClineShapeCWTlTrDParamRec(ClineShapeCWTlTrDParamRec& p):
					r_(p.r_){
  	SetPointerToDerivedSelf();
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
  	c_=p.c_; w_=p.w_;
  	thetaL_=p.thetaL_; thetaR_=p.thetaR_;
  	zL_=p.zL_; zR_=p.zR_;
  	xL_=p.xL_; xR_=p.xR_;
  	D_=p.D_; lnLDiseq_=p.lnLDiseq_;
	}



ParamRec* ClineShapeCWTlTrDParamRec::Duplicate(void){
	ClineShapeCWTlTrDParamRec* copy = NEW ClineShapeCWTlTrDParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrDParamRec::Duplicate()


ClineShapeCWTlTrDParamRec* ClineShapeCWTlTrDParamRec::_Duplicate(void){
	ClineShapeCWTlTrDParamRec* copy = NEW ClineShapeCWTlTrDParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrDParamRec::_Duplicate()


void ClineShapeCWTlTrDParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineShapeCWTlTrDParamRec"<<endl;
  	}//end ClineShapeCWTlTrDParamRec::SetPointerToDerivedSelf()




long double ClineShapeCWTlTrDParamRec::r(void){
	return r_;}


void ClineShapeCWTlTrDParamRec::r(long double rr){
	r_ = rr;}
	
	
long double ClineShapeCWTlTrDParamRec::distanceBarrierL(void){//barrier to introgression in distance units
    //derivatives found using Mathematica
    long double derivFactor = (long double)4.0 * sqrt(thetaL_) * (xL_ - (c_ + zL_)) / w_;
    long double dPLdXatxL, beta;
    if ((w_ ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp(100.0);}//huge and undefined
    if ((w_ ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp((long double)100.0);}//huge and undefined
    if (derivFactor <= (long double)100.0){
			dPLdXatxL = exp(derivFactor) * sqrt(thetaL_) / w_;}
    	else{
			dPLdXatxL = exp((long double)100.0);}

    if(dPLdXatxL == (long double)0.0){
			return exp((long double)100.0);}
		else{
    		long double dP = DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPLdXatxL;}
	return beta;
	}//end ClineShapeCWTlTrDParamRec::distanceBarrierL()
 


long double ClineShapeCWTlTrDParamRec::distanceBarrierR(void){//barrier to introgression in distance units
    //derivatives found using Mathematica
    long double derivFactor = (long double)-4.0 * sqrt(thetaR_) * (xR_ - (c_ - zR_)) / w_;
    long double dPRdXatxR, beta;
    if ((w_ ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp((long double)100.0);}//huge and undefined
    if ((w_ ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp((long double)100.0);}//huge and undefined
    if (derivFactor <= (long double)100.0){
			dPRdXatxR = exp(derivFactor) * sqrt(thetaR_) / w_;}
    	else{
			dPRdXatxR = exp((long double)100.0);}

    if(dPRdXatxR == (long double)0.0){
			return exp((long double)100.0);}
		else{
    		long double dP = DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPRdXatxR;}
	return beta;
	}//end ClineShapeCWTlTrDParamRec::distanceBarrierR()
 


int ClineShapeCWTlTrDParamRec::ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldCenter = c_;
	ClineShapeCWTlTrDParamRec* lower = (ClineShapeCWTlTrDParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineShapeCWTlTrDParamRec* upper = (ClineShapeCWTlTrDParamRec*) upperLimits->pointerToDerivedSelf;
	do{
		c_ = ran.NormDev(oldCenter, sigma);
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = RJunction(upper->xR_);}
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrDParamRec::ChangeCenter(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrDParamRec::ChangeThetaL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldThL = thetaL_;
	ClineShapeCWTlTrDParamRec* lower = (ClineShapeCWTlTrDParamRec*) lowerLimits->pointerToDerivedSelf;
	do{
		thetaL_ = ran.NormDev(oldThL, sigma);
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrDParamRec::ChangeThetaL(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrDParamRec::ChangeThetaR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldThR = thetaR_;
	ClineShapeCWTlTrDParamRec* upper = (ClineShapeCWTlTrDParamRec*) upperLimits->pointerToDerivedSelf;
	do{
		thetaR_ = ran.NormDev(oldThR, sigma);
		validChange = RJunction(upper->xR_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrDParamRec::ChangeThetaR(Ran3&, double, ParamRec*, ParamRec*)



int ClineShapeCWTlTrDParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrDParamRec* deviations = (ClineShapeCWTlTrDParamRec*) paramDeviations->pointerToDerivedSelf;
	int success = 0;
	switch (paramNum){
	 case 0:
	 	  success =  ChangeCenter(ran, deviations->center()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	  success =  ChangeWidth(ran, deviations->width()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 2:
	 	  success =  ChangeThetaL(ran, deviations->thetaL()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 3:
	 	  success =  ChangeThetaR(ran, deviations->thetaR()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 4:
	 	  success =  ChangeZL(ran, deviations->zL()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 5:
	 	  success =  ChangeZR(ran, deviations->zR()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 6:
	 	  success =  ChangeD(ran, deviations->D()*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 7-parameter cline model");
		aError(errstr);
	 	}
	return  success;
	}//end ClineShapeCWTlTrDParamRec::ChangeParam(int,Ran3&,ParamRec*,ParamRec*,ParamRec*)


int ClineShapeCWTlTrDParamRec::ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldWidth = w_;
	ClineShapeCWTlTrDParamRec* lower = (ClineShapeCWTlTrDParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineShapeCWTlTrDParamRec* upper = (ClineShapeCWTlTrDParamRec*) upperLimits->pointerToDerivedSelf;
	do{
		w_ = ran.NormDev(oldWidth, sigma);
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = RJunction(upper->xR_);}
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrDParamRec::ChangeWidth(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrDParamRec::ChangeZL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldZL = zL_;
	ClineShapeCWTlTrDParamRec* lower = (ClineShapeCWTlTrDParamRec*) lowerLimits->pointerToDerivedSelf;
	do{
		zL_ = ran.NormDev(oldZL, sigma);
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrDParamRec::ChangeZL(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrDParamRec::ChangeZR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldZR = zR_;
	ClineShapeCWTlTrDParamRec* upper = (ClineShapeCWTlTrDParamRec*) upperLimits->pointerToDerivedSelf;
	do{
		zR_ = ran.NormDev(oldZR, sigma);
		validChange = RJunction(upper->xR_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrDParamRec::ChangeZR(Ran3&, double, ParamRec*, ParamRec*)


long double ClineShapeCWTlTrDParamRec::DispersalSigma(void){
    if (D_ > 0.0){
		long double sigma = sqrt((D_ * r_ * w_ * w_) / ((long double)1.0 + r_));
		return sigma;}
	return (long double)0.0;//default
	}//end ClineShapeCWTlTrDParamRec::DispersalSigma()



void ClineShapeCWTlTrDParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	c_= i.c_;
	w_ = i.w_;
	thetaL_ = i.thetaL_;
	thetaR_ = i.thetaR_;
	zL_ = i.zL_;
	zR_ = i.zR_;
	LJunction(i.leftEnd_);
	RJunction(i.rightEnd_);
	D_ = i.D_;
	lnLDiseq_ = 0.0;	
	}//end ClineShapeCWTlTrDParamRec::Initialize(ClineFitInitializer&)


int ClineShapeCWTlTrDParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrDParamRec* lower = (ClineShapeCWTlTrDParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineShapeCWTlTrDParamRec* upper = (ClineShapeCWTlTrDParamRec*) upperLimits->pointerToDerivedSelf;
	if(c_ <lower->c_){return 0;}
		else if (c_ > upper->c_){return 0;}
		else {}
	if(w_ <= 0.0){return 0;}
		else if (w_ > upper->w_){return 0;}
		else {}
	if(thetaL_ <lower->thetaL_){return 0;}
		else if (thetaL_ > upper->thetaL_){return 0;}
		else {}
	if(thetaR_ <lower->thetaR_){return 0;}
		else if (thetaR_ > upper->thetaR_){return 0;}
		else {}
	if(zL_ <lower->zL_){return 0;}
		else if (zL_ > upper->zL_){return 0;}
		else {}
	if(zR_ <lower->zR_){return 0;}
		else if (zR_ > upper->zR_){return 0;}
		else {}
	if((xL_ > xR_)||(xL_>c_)||(xR_<c_)){return 0;}
		else if(xL_ < lower->xL_){return 0;}
		else if(xR_ > upper->xR_){return 0;}
		else {}
	if(D_ <lower->D_){return 0;}
		else if (D_ > upper->D_){return 0;}
		else {}
	return 1;
	}//end ClineShapeCWTlTrDParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineShapeCWTlTrDParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	switch (paramNum){
	 case 0:
	 	 c_ *= factor;
	 	 break;
	 case 1:
	 	 w_ *= factor;
	 	 break;
	 case 2:
	 	 thetaL_ *= factor;
	 	 break;
	 case 3:
	 	 thetaR_ *= factor;
	 	 break;
	 case 4:
	 	 zL_ *= factor;
	 	 break;
	 case 5:
	 	 zR_ *= factor;
	 	 break;
	 case 6:
	 	 D_ *= factor;
	 	 break;
	 default:
		aString errstr("bug: can't rescale param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 7-parameter cline model");
		aError(errstr);
	 	}
	}//ClineShapeCWTlTrDParamRec::MultiplyParam(int, double)



long double ClineShapeCWTlTrDParamRec::NumSelectedLoci(void){
	long double s = s_star_vs_Hets();
    if (s == (long double)0.0){
		return (long double)0.0;}//no loci under selection if no selection
	long double ww = WhybridOverWpure();
    if ((s >= (long double)1.0) || (ww <= (long double)0.0)){
		return (long double)1.0e20;}//a really high number if fitness=0
    double nLocSel = (long double)2.0 * ln(ww) / ln((long double)1.0 - s);
    return nLocSel;
    }//end ClineShapeCWTlTrDParamRec::NumSelectedLoci()





int ClineShapeCWTlTrDParamRec::NumParams(void){
	return 7;
	}//end ClineShapeCWTlTrDParamRec::NumParams()


int ClineShapeCWTlTrDParamRec::NumDerivedParams(void){
	return 5 + 14;
	}//end ClineShapeCWTlTrDParamRec::NumParams()



aString* ClineShapeCWTlTrDParamRec::ParamName(int paramNum){
	//returns name of parameter; first varying params, then derived params
	switch (paramNum){
	 case 0:
	 	 return NEW aString("center");
	 	 break;
	 case 1:
	 	 return NEW aString("width");
	 	 break;
	 case 2:
	 	 return NEW aString("thetaL");
	 	 break;
	 case 3:
	 	 return NEW aString("thetaR");
	 	 break;
	 case 4:
	 	 return NEW aString("zL");
	 	 break;
	 case 5:
	 	 return NEW aString("zR");
	 	 break;
	 case 6:
	 	 return NEW aString("D");
	 	 break;
	 case 7:
	 	 return NEW aString("xL");
	 	 break;
	 case 8:
	 	 return NEW aString("xR");
	 	 break;
	 case 9:
	 	 return NEW aString("p[xL]");
	 	 break;
	 case 10:
	 	 return NEW aString("p[xR]");
	 	 break;
	 case 11:
	 	 return NEW aString("deltaP");
	 	 break;
	 case 12:
	 	 return NEW aString("sigma");//dispersal
	 	 break;
	 case 13:
	 	 return NEW aString("s*[hets]");
	 	 break;
	 case 14:
	 	 return NEW aString("s*[ecotone]");
	 	 break;
	 case 15:
	 	 return NEW aString("s*[dom&ecotone]");
	 	 break;
	 case 16:
	 	 return NEW aString("W[hybrid]/[W[pure]");
	 	 break;
	 case 17:
	 	 return NEW aString("S");
	 	 break;
	 case 18:
	 	 return NEW aString("barrierL[dist]");
	 	 break;
	 case 19:
	 	 return NEW aString("barrierR[dist]");
	 	 break;
	 case 20:
	 	 return NEW aString("barrierL[time]");
	 	 break;
	 case 21:
	 	 return NEW aString("barrierR[time]");
	 	 break;
	 case 22:
	 	 return NEW aString("sL[marker]");
	 	 break;
	 case 23:
	 	 return NEW aString("sR[marker]");
	 	 break;
	 case 24:
	 	 return NEW aString("num loci selected");
	 	 break;
	 case 25:
	 	 return NEW aString("s per selected locus");
	 	 break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 7-parameter cline model");
		aError(errstr);
	 	}
	return NEW aString();
	}//end ClineShapeCWTlTrDParamRec::ParamName(int )



void ClineShapeCWTlTrDParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWTlTrParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(lnLDiseq_,6,15);
	dataline.cat(sepchar);
	dataline.cat(DispersalSigma(),6,15);
	dataline.cat(sepchar);
	dataline.cat(s_star_vs_Hets(),2,15);
	dataline.cat(sepchar);
	dataline.cat(s_star_Ecotone(),2,15);
	dataline.cat(sepchar);
	dataline.cat(s_star_DomEcotone(),2,15);
	dataline.cat(sepchar);
	dataline.cat(WhybridOverWpure(),6,15);
	dataline.cat(sepchar);
	dataline.cat(S(),6,15);
	dataline.cat(sepchar);
	dataline.cat((double)distanceBarrierL(),9,15);
	dataline.cat(sepchar);
	dataline.cat((double)distanceBarrierR(),9,15);
	dataline.cat(sepchar);
	dataline.cat((double)timeBarrierL(),9,15);
	dataline.cat(sepchar);
	dataline.cat((double)timeBarrierR(),9,15);
	dataline.cat(sepchar);
	dataline.cat(s_perMarkerLocusL(),2,15);
	dataline.cat(sepchar);
	dataline.cat(s_perMarkerLocusR(),2,15);
	dataline.cat(sepchar);
	dataline.cat(NumSelectedLoci(),6,15);
	dataline.cat(sepchar);
	dataline.cat(s_perReproIsolationLocus(),2,15);
	}//end ClineShapeCWTlTrDParamRec::ParamString(aString&, char)



long double ClineShapeCWTlTrDParamRec::ParamValue(int paramNum){
	//returns value of parameter; first varyng params, then derived params
	switch (paramNum){
	 case 0:
	 	 return center();
	 	 break;
	 case 1:
	 	 return width();
	 	 break;
	 case 2:
	 	 return thetaL();
	 	 break;
	 case 3:
	 	 return thetaR();
	 	 break;
	 case 4:
	 	 return zL();
	 	 break;
	 case 5:
	 	 return zR();
	 	 break;
	 case 6:
	 	 return D();
	 	 break;
	 case 7:
	 	 return xL();
	 	 break;
	 case 8:
	 	 return xR();
	 	 break;
	 case 9:
	 	 return B_L(0);//base class assumes average trait
	 	 break;
	 case 10:
	 	 return B_R(0);//base class assumes average trait
	 	 break;
	 case 11:
	 	 return DeltaP(0);//base class assumes average trait
	 	 break;
	 case 12:
	 	 return DispersalSigma();//dispersal
	 	 break;
	 case 13:
	 	 return s_star_vs_Hets();
	 	 break;
	 case 14:
	 	 return s_star_Ecotone();
	 	 break;
	 case 15:
	 	 return s_star_DomEcotone();
	 	 break;
	 case 16:
	 	 return WhybridOverWpure();
	 	 break;
	 case 17:
	 	 return S();
	 	 break;
	 case 18:
	 	 return distanceBarrierL();
	 	 break;
	 case 19:
	 	 return distanceBarrierR();
	 	 break;
	 case 20:
	 	 return timeBarrierL();
	 	 break;
	 case 21:
	 	 return timeBarrierR();
	 	 break;
	 case 22:
	 	 return s_perMarkerLocusL();
	 	 break;
	 case 23:
	 	 return s_perMarkerLocusR();
	 	 break;
	 case 24:
	 	 return NumSelectedLoci();
	 	 break;
	 case 25:
	 	 return s_perReproIsolationLocus();
	 	 break;
	 default:
		aString errstr("bug: can't return value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 7-parameter cline model");
		aError(errstr);
	 	}
	return 0.0;
	}//end ClineShapeCWTlTrDParamRec::ParamValue(int )


void ClineShapeCWTlTrDParamRec::Print(ostream& outFile, long rep, aString& message){
	ClineShapeCWTlTrParamRec::Print(outFile, rep, message);
	aString outStr("D: ");
	outStr.cat(D(),2,15);
	outStr.separator(2);
	outStr.cat("lnLDiseq: ");
	outStr.cat(lnLDiseq(),6,15);
	outFile<<outStr<<endl;
	}//ClineShapeCWTlTrDParamRec::Print(ostream&, long, aString&)


long double ClineShapeCWTlTrDParamRec::S(void){
    if (WhybridOverWpure() == (long double)0.0){
		return (long double)200.0;}//a high number
    double bigS = (long double)-2.0 * ln(WhybridOverWpure());
    return bigS;
    }//end ClineShapeCWTlTrDParamRec::S()


void ClineShapeCWTlTrDParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter; only for varying parameters
	switch (paramNum){
	 case 0:
	 	 center(newVal);
	 	 break;
	 case 1:
	 	 width(newVal);
	 	 break;
	 case 2:
	 	 thetaL(newVal);
	 	 break;
	 case 3:
	 	 thetaR(newVal);
	 	 break;
	 case 4:
	 	 zL(newVal);
	 	 break;
	 case 5:
	 	 zR(newVal);
	 	 break;
	 case 6:
	 	 D(newVal);
	 	 break;
	 default:
		aString errstr("bug: can't set value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 7-parameter cline model");
		aError(errstr);
	 	}
	}//end ClineShapeCWTlTrDParamRec::SetParamValue(int,long double)


long double ClineShapeCWTlTrDParamRec::s_perMarkerLocusL(void){
	//direct selection on marker loci in left tail of cline
	//assuming genetic background incompatibilities
	if(w_==0.0){return (long double)0.0;}//undefined
    long double s = thetaL_ * s_star_vs_Hets();
    return s;
    }//end ClineShapeCWTlTrDParamRec::s_perMarkerLocusL()


long double ClineShapeCWTlTrDParamRec::s_perMarkerLocusR(void){
	//direct selection on marker loci in right tail of cline
	//assuming genetic background incompatibilities
	if(w_==0.0){return (long double)0.0;}//undefined
    long double s = thetaR_ * s_star_vs_Hets();
    return s;
    }//end ClineShapeCWTlTrDParamRec::s_perMarkerLocusL()


long double ClineShapeCWTlTrDParamRec::s_perReproIsolationLocus(void){
	//average selection on a locus contributing to reproductive isolation
	long double n = NumSelectedLoci();
	 if (n == 0.0){
			return (long double) 0.0;}
    return S() / n;
    }//end ClineShapeCWTlTrDParamRec::s_perReproIsolationLocus(void)




long double ClineShapeCWTlTrDParamRec::s_star_vs_Hets(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming genetic background incompatibilities
	if(w_==0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)2.0 * DispersalSigma() / w_);
    return s;
    }//end ClineShapeCWTlTrDParamRec::s_star_vs_Hets()


long double ClineShapeCWTlTrDParamRec::s_star_Ecotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone
	if(w_==0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.732 * DispersalSigma() / w_);
    return s;
    }//end ClineShapeCWTlTrDParamRec::s_star_Ecotone()


long double ClineShapeCWTlTrDParamRec::s_star_DomEcotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone on a dominant marker
	if(w_==0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.782 * DispersalSigma() / w_);
    return s;
    }//end ClineShapeCWTlTrDParamRec::s_star_DomEcotone()




long double ClineShapeCWTlTrDParamRec::timeBarrierL(void){
	long double d = DispersalSigma();
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL();
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);
    return timeL;
}//end ClineShapeCWTlTrDParamRec::timeBarrierL()


long double ClineShapeCWTlTrDParamRec::timeBarrierR(void){
	long double d = DispersalSigma();
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR();
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);
    return timeR;
}//end ClineShapeCWTlTrDParamRec::timeBarrierR()


void ClineShapeCWTlTrDParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWTlTrParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("lnLDiseq");
	dataline.cat(sepchar);
	dataline.cat("D");
	dataline.cat(sepchar);
	dataline.cat("sigma[dispersal]");
	dataline.cat(sepchar);
	dataline.cat("s*[hets]");
	dataline.cat(sepchar);
	dataline.cat("s*[ecotone]");
	dataline.cat(sepchar);
	dataline.cat("s*[dom_ecotone]");
	dataline.cat(sepchar);
	dataline.cat("Whyb/Wpure");
	dataline.cat(sepchar);
	dataline.cat("S");
	dataline.cat(sepchar);
	dataline.cat("barrierL[dist]");
	dataline.cat(sepchar);
	dataline.cat("barrierR[dist]");
	dataline.cat(sepchar);
	dataline.cat("barrierL[time]");
	dataline.cat(sepchar);
	dataline.cat("barrierR[time]");
	dataline.cat(sepchar);
	dataline.cat("sL_per_marker_loc");
	dataline.cat(sepchar);
	dataline.cat("sR_per_marker_loc");
	dataline.cat(sepchar);
	dataline.cat("n_loci_selected");
	dataline.cat(sepchar);
	dataline.cat("sR_per_repro_isol_loc");
	}//end ClineShapeCWTlTrDParamRec::TitleString(aString&, char)


long double ClineShapeCWTlTrDParamRec::WhybridOverWpure(void){
	if(w_==0){return (long double)0.0;}//undefined
	long double beta = (distanceBarrierL() + distanceBarrierR()) / (long double)2.0;
    if ((beta / w_ > (long double)3000.0) || (beta / w_ == (long double)0.0)){
		return (long double)0.0;}
    double wratio = exp(- r_ * (ln(beta / w_)));
    return wratio;
	}//end ClineShapeCWTlTrDParamRec::WhybridOverWpure()


void ClineShapeCWTlTrDParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineShapeCWTlTrParamRec
	ClineShapeCWTlTrDParamRec* pp = (ClineShapeCWTlTrDParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
  	c_ = pp->c_;
  	w_ = pp->w_;
	thetaL_ = pp->thetaL_;
	thetaR_ = pp->thetaR_;
	zL_ = pp->zL_;
	zR_ = pp->zR_;
	xL_ = pp->xL_;
	xR_ = pp->xR_;
	D_ = pp->D_;
	lnLDiseq_ = pp->lnLDiseq_;
	r_ = pp->r_;
	}//end  ClineShapeCWTlTrDParamRec::operator=(ParamRec&)





int	operator==(ClineShapeCWTlTrDParamRec& p, ClineShapeCWTlTrDParamRec& q){
	return ((p.lnL_ == q.lnL_) &&
			(p.c_ == q.c_) &&
			(p.w_ == q.w_) &&
			(p.thetaL_ == q.thetaL_) &&
			(p.thetaR_ == q.thetaR_) &&
			(p.zL_ == q.zL_) &&
			(p.zR_ == q.zR_) &&
			(p.D_ == q.D_) &&
			(p.lnLDiseq_ == q.lnLDiseq_) &&
			(p.r_ == q.r_)
			 );
	}//end operator==(ClineShapeCWTlTrDParamRec&, ClineShapeCWTlTrDParamRec&)


int	operator!=(ClineShapeCWTlTrDParamRec& p, ClineShapeCWTlTrDParamRec& q){
	return !(p==q);
	}//end operator!=(ClineShapeCWTlTrDParamRec&, ClineShapeCWTlTrDParamRec&)



#endif

/*


 procedure SetDerivedVariables (var paramSet: paramRec; harMeanRecomb: extended);
  var
   deltaP, dPLdXatxL, dPRdXatxR, rVal, lVal: extended;
 begin
  with paramSet do
   begin
    rVal := PonCline(xR, center, w, xL, xR, thetaL, thetaR, bottomL, topR, barrierL, barrierR, maxScore);
    lVal := PonCline(xL, center, w, xL, xR, thetaL, thetaR, bottomL, topR, barrierL, barrierR, maxScore);
    deltaP := abs(rVal - lVal);
    if (4.0 * Sqrt(thetaL) * (xL - (center + barrierL)) / w <= -100.0) then
     dPLdXatxL := Exp(-100.0)
    else if (4.0 * Sqrt(thetaL) * (xL - (center + barrierL)) / w <= 100.0) then
     dPLdXatxL := 4.0 * Exp(4.0 * Sqrt(thetaL) * (xL - (center + barrierL)) / w) * Sqrt(thetaL) / w
    else		{derivatives found using Mathematica}
     dPLdXatxL := Exp(100.0);

    if (4.0 * Sqrt(thetaR) * (xR - (center - barrierR)) / w) <= -100.0 then
     dPRdXatxR := Exp(100.0)
    else if (4.0 * Sqrt(thetaR) * (xR - (center - barrierR)) / w) <= 100.0 then
     dPRdXatxR := 4.0 * Sqrt(thetaR) / (w * Exp(4.0 * Sqrt(thetaR) * (xR - (center - barrierR)) / w))
    else
     dPRdXatxR := Exp(-100.0);

    if dPLdXatxL = 0.0 then
     beta1 := Exp(100.0)
    else
     beta1 := deltaP / dPLdXatxL;
    if dPRdXatxR = 0.0 then
     beta2 := Exp(100.0)
    else
     beta2 := deltaP / dPRdXatxR;

    if D < 0.0 then
     sigma := 0.0
    else
     sigma := sqrt((D * meanRecomb * w * w) / (1 + meanRecomb));
    if sigma = 0.0 then
     begin
      timeL := 1.0e20;	{basically forever if no dispersal}
      timeR := 1.0e20;
     end
    else if (beta1 / sigma > 1.0e40) then
     begin
      timeL := 1.0e20;	{basically forever if beta/sigma too high}
      timeR := 1.0e20;
     end
    else
     begin
      timeL := sqr(beta1 / sigma);
      timeR := sqr(beta2 / sigma);
     end;	{end if sigma=0 else if ... else...}
    if ((((beta1 + beta2) / 2.0) / w > 3000.0) or (((beta1 + beta2) / 2.0) / w = 0.0)) then
     WhybridOverWpure := 0.0
    else
     WhybridOverWpure := exp(-harMeanRecomb * (ln((beta1 + beta2) / 2.0) / w));
    if WhybridOverWpure = 0.0 then
     bigS := 200.0	{a high number}
    else
     bigS := -2 * ln(WhybridOverWpure);
    sVsHets := sqr(2.0 * sigma / w);
    sStarEcotone := sqr(1.732 * sigma / w);
    sStarDomEcotone := sqr(1.782 * sigma / w);
    if sVsHets = 0.0 then
     nLocSel := 0.0		{no loci under selection if no selection}
    else if ((sVsHets >= 1.0) or (WhybridOverWpure <= 0.0)) then
     nLocSel := 1.0e20		{a really high number if selection=0}
    else
     nLocSel := 2.0 * ln(WhybridOverWpure) / ln(1 - sVsHets);
    sperMarkerL := thetaL * sVsHets;
    sperMarkerR := thetaR * sVsHets;
    if nLocSel = 0.0 then
     sVsHetsRI := 0.0
    else
     sVsHetsRI := bigS / nLocSel;
   end;	{end with paramSet}
 end;	{end procedure SetDerivedVariables}



*/
