/******************************************************
*
*	ClineShapeCWTlTrTBQ2ParamRec.h
*
*	Quantitative trait parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	These parameters vary for each trait, so the mean y-axis value of the cline also
*	varies for each trait on each side.  Cline regions may be defined as {left, center,right} 
*	or each population separately; the former is the typical cline estimation method.
*	This class includes estimation of a covariance matrix among traits for each cline region,
*	allowing immediate estimation of disequilibrium (in the center) and the cascade of derived cline parameters.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrTBQ2ParamRech
#define ClineShapeCWTlTrTBQ2ParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineShapeCWTlTrTBParamRec.h"
#include "aVector.cpp"
#include "aMatrix.cpp"
#include "MatrixList.h"
#include "ClineFitInitializer.h"
#include "MemoryManager.h"

class ClineShapeCWTlTrTBQ2ParamRec;
class ClineShapeCWTlTrTBQ2ParamRec: virtual public ParamRec{
  public:
  	ObjList<ParamRec> traitParamSet_;//a parameter set for each trait; actually of type ClineShapeCWTlTrTBParamRec
  	long double r_;//harmonic mean recombination rate
  	int covMatrixListType_;// >3 for 'populations', 3 for 'cline regions' = left, center, right
  	MatrixList covMatrixList_;//trait variance-covariance matrices for each population or region; index [pop][trait][trait]
  	MatrixList invCovMatrixList_;//inverses of variance-covariance matrices ; index [pop][trait][trait]
  	aList<long double> detCovMatrixList_;//determinants of variance-covariance matrices; index [pop]
  	aList<int> invCovMatrixFlags_;//flags; index [pop]
  	aList<int> detCovMatrixFlags_;//flags; index [pop]
  	int nparams_, nshparams_, ndparams_;//num overall varying parameters, num shape parameters, num derived params
  	ObjList<aList<int> > paramIdentities_;//indexed [param][trait]; parameters constrained to change together;param off=0
	aList<int> varyingParams_;//indexed [param]; number of differently constrained parameters per trait
			//param order: c_[0],w_[1],thL_[2],thR_[3],zL_[4],zR_[5],leftVal_[6],rightVal_[7]
			//covariance matrix params left out of this --> all must vary independently anyway
			/*e.g., cline with 3 traits:
			If all centers were constrained to a common value, 
					paramIdentities_[0][0] = paramIdentities_[0][1] = paramIdentities_[0][2] = 1; 
					varyingParams_[0]=1;//can't be 0 unless center not estimated at all
			If the first two centers were constrained to a common value, 
					paramIdentities_[0][0] = paramIdentities_[0][1] = 1; paramIdentities_[0][2] = 2; 
					varyingParams_[0]=2;//not 0-indexed here
			etc.
			*/
  	long double D_;//disequilibrium if >1 trait
  	int Dcalculated_;//flag
  public:
  	ClineShapeCWTlTrTBQ2ParamRec(void);
  	ClineShapeCWTlTrTBQ2ParamRec(ClineFitInitializer& i);
  	ClineShapeCWTlTrTBQ2ParamRec(ClineShapeCWTlTrTBQ2ParamRec& p);
  	virtual ~ClineShapeCWTlTrTBQ2ParamRec(void){
//  			pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineShapeCWTlTrTBQ2ParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double B_L(int trait);//expected phenotype at cline location xL
	virtual long double B_R(int trait);//expected phenotype at cline location xR
	virtual long double center(int trait);//returns value
	virtual void 		center(int trait, long double value);//sets value
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeCenter(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeCovMatrix(Ran3& ran, int paramNum, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, double weight=1.0);
	virtual int 		ChangeLeftScore(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeRightScore(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeThetaL(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeThetaR(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeWidth(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeZL(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual int 		ChangeZR(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight=1.0);
	virtual long double Cov(int trait1, int trait2, int region);
	virtual void 		CovParamCoords(int paramNum, int& region, int& atTrait1, int& atTrait2);
								//replaces value of atParam and fills traitList with identities of co-estimated trait parameters
	virtual long double CovTraitDeterminant(int region);
	virtual long double D(void);//returns disequilibrium from covariance structure
	virtual long double DeltaP(int trait);//the trait score step across the center region, xL to xR
	virtual long double DispersalSigma(void);//from covariances; averaged over trait pairs
	virtual long double distanceBarrierL(int trait);//barrier to introgression from left to right in distance units
	virtual long double distanceBarrierR(int trait);//barrier to introgression from right to left in distance units
	virtual long double ExpectedValue(aList<long double>& coords);//also appends correct variance term to coords
	virtual void 		Initialize(ClineFitInitializer& i);
	virtual aMatrix<long double>* InverseCovTraitMatrix(int region);
	virtual long double leftVal(int trait);//returns value
	virtual void 		leftVal(long double value,int trait);//sets value
	virtual int 		LJunction(int trait, double leftEnd);//sets xL[trait] & returns 0 if the cline sections don't meet
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumCovMatrixParams(void);//number of varying params estimated in covariance matrix
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters
	virtual int 		NumParams(void);//number of varying parameters
	virtual int 		NumShapeParams(void);//number of varying parameters affecting shapes of trait clines
	virtual long double NumSelectedLoci(void);//assumes many loci and selection via genetic background
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
			long double r(void);//returns r_
			void 		r(long double newVal);//sets r_
	virtual int			regions(void){return covMatrixList_.len();}
	virtual long double rightVal(int trait);//returns value
	virtual void 		rightVal(long double value,int trait);//sets value
	virtual int 		RJunction(int trait, double rightEnd);//sets xR[trait] & returns 0 if the cline sections don't meet
	virtual long double S(void);//effective selection against hybrids in center as if hybrids were single-locus heterozygotes
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		ShapeParamCoords(int paramNum, int& atParam, aList<int>& traitList);
								//replaces value of atParam and fills traitList with identities of co-estimated trait parameters
	virtual long double s_perMarkerLocusL(int trait);//ave. direct selection on marker loci in left tail of cline
	virtual long double s_perMarkerLocusR(int trait);//ave. direct selection on marker loci in right tail of cline
	virtual long double s_perReproIsolationLocus(void);//average selection on a locus contributing to reproductive isolation
	
						//indirect & direct selection on marker loci in middle of cline
	virtual long double s_star_Ecotone(void);//assuming selection across an ecotone
	virtual long double s_star_vs_Hets(void);//assuming genetic background incompatibilities

	virtual long double thetaL(int trait);//returns value
	virtual void 		thetaL(int trait, long double value);//sets value
	virtual long double thetaR(int trait);//returns value
	virtual void 		thetaR(int trait, long double value);//sets value
	virtual long double timeBarrierL(int trait);
	virtual long double timeBarrierR(int trait);
	virtual long double	traitCov(int trait1, int trait2, int region);//returns covariance for region
//	virtual double		traitCov(int trait1, int trait2, double x);//returns variance for location x
	virtual int			traits(void);
	virtual long double	traitVar(int trait,int region);
//	virtual double		traitVar(int trait, double x);//returns variance for location x
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual	long double WhybridOverWpure(void);//ratio in cline center
	virtual long double width(void);//returns mean value
	virtual long double width(int trait);//returns value
	virtual void 		width(int trait, long double value);//sets value
	virtual long double xL(int trait);//returns value
	virtual void 		xL(int trait, long double value);//sets value -- useful only for setting limits 
	virtual long double xR(int trait);//returns value
	virtual void 		xR(int trait, long double value);//sets value -- useful only for setting limits
	virtual long double zL(int trait);//returns value
	virtual void 		zL(int trait, long double value);//sets value
	virtual long double zR(int trait);//returns value
	virtual void 		zR(int trait, long double value);//sets value
	virtual aVector<long double>*
						ZonCline(long double x);//x=location; returns new vector
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWTlTrTBQ2ParamRec& p, ClineShapeCWTlTrTBQ2ParamRec& q);
	friend int	operator!=(ClineShapeCWTlTrTBQ2ParamRec& p, ClineShapeCWTlTrTBQ2ParamRec& q);

	};//end class ClineShapeCWTlTrTBQParamRec



#endif