/***************************
*
*
*	Program:  ClineFit
*
*	This program fits genetic marker data and phenotypic data
*	to the Barton cline shape model using a likelihood function maximized
*	via a Metropolis-Hastings algorithm.
*
**************************************************/


#include "Element.cpp"
#include "aList.cpp"
#include "aTime.h"
#include <string>
#include <sstream>
#include "aMatrix.cpp"
#include "MatrixList.h"
#include "StringClass.h"
#include "RandomList.h"
#include "ClineFitGlobals.h"
#include "HarmonicMeanDiseq.h"
//#include "HaplotypeData.h"
#include "PopulationGeneticData.h"
#include "PopulationPhenotypicData.h"
#include "FstatisticAnalysis.h"
#include "ParamRec.h"
#include "ClineData.h"
#include "ClineGeneticData.h"
#include "ClinePhenotypicData.h"
#include "MetropolisHastings.h"
#include "ClineFitFunctions.h"
#include "ClineFitInitializer.h"
#include "FitClineShapes.h"
#include "MemoryManager.h"
#include <fstream>
#include <iostream>

int main(void){using namespace std;


	int OK=0, legal=1, useDefaultSettings=1;	
	char yn;
	long longseed, randomNumberSeed = 50;
	bool firsttime=true;//used for opening some files
	aTime *watch = NEW aTime();

	int i, j, loc, p, pop, pto, pid, r, t, tr, tr0, trnum, currentState, listedSoFar, firstTrait;
	aString lineout;
	aList<int> traitsToSkip;
	//THESE ARE FOR DOING ONLY ONE DATA SET
//    aString dataFileName = "sample cline allozyme data";
    aString dataFileName = "sampleClineAllozymeData";
    aString outputFileName = "debug results cline fit";
//    aString dataFileName = "WestBIOSYS";
//    aString outputFileName = "WestBIOSYS results";
//    aString dataFileName = "Napi-Bryo mtDNA data";
//    aString outputFileName = "Napi-Bryo mtDNA results";
    aString morphDataFileName = "sample cline morph data";

	int genotypicDataUsed = true;//true or false
	int mtDNAdataUsed = false;//true or false -- not supported yet
	int morphologicalDataUsed = false;//true or false
	int dominantMarkerDataUsed = false;//not supported yet

	int testingFitToGivenShape = false;//not implemented for the new on/off switches yet
	int enteringStartingValues = false;//not implemented for the new on/off switches yet

	long savedReps = 2000;//2000 -- set to lower numbers if you're getting a new data set going
	long repsPerAnnealStep = 25;//250; 1000;in changing parameters
	int repsBetweenSaves = 5;//default 50; good replicates to skip when saving parameters
	
	int paramsToVary = 9;//2,4,6,7,8,9; must use 8 with quantitative traits // 4, 5, 9 in future versions
				//2: center, width  (shape only)
				//4: center, width, left mean score, right mean score  (shape only)
				//6: center, width, thetaL, thetaR, zL, zR (shape only)
				//7: center, width, thetaL, thetaR, zL, zR, D (shape, diseq, dispersal, selection etc.)
				//8: center, width, thetaL, thetaR, zL, zR, p[L], p[R]  (single-locus shape only)
				//9: center, width, thetaL, thetaR, zL, zR, p[L], p[R], D  (shape, diseq, dispersal, selection etc.)



	int interface=1;//toggle for debugging
	if (interface){
		cout<<"\tThis program fits a Barton cline-shape model to data from a transect through a hybrid zone. \
The data may consist of the genotypes of individuals at 1 or more loci. The cline may have up to 9 core parameters,"<<endl;
cout<<"\tcenter -- midpoint location of the cline"<<endl;
cout<<"\twidth  -- inverse of the maximum slope at the midpoint"<<endl;
cout<<"\tthetaL -- exponential slope of the introgression tail on the left side"<<endl;
cout<<"\tthetaR -- exponential slope of the introgression tail on the right side"<<endl;
cout<<"\tzL     -- asymptote of the exponentially decaying introgression tail for the left side"<<endl;
cout<<"\tzR     -- asymptote of the exponentially decaying introgression tail for the right side"<<endl;
cout<<"\tpL     -- asymptotic marker frequency on the left end of the cline; default pL=0.0"<<endl;
cout<<"\tpR     -- asymptotic marker frequency on the right end of the cline; default pR=1.0"<<endl;
cout<<"\tD      -- gametic disequilibrium in the central region of the cline; for multi-trait data only."<<endl;
cout<<"\tSufficient control is provided so that the user can fit a variety of special-case models and thereby \
test many hypotheses about the significance of particular parameter combinations.  The user can \
turn off any of these core parameters individually for the entire model, or turn them off \
separately for each trait.  Parameters of each core type can be shared over user-defined sets of traits, \
to test hypotheses about which markers tend to covary."<<endl;
cout<<"\tIf D is estimated, then several parameters related to dispersal and selection strength will be estimated as well."<<endl;
cout<<"\tCurrently, only data from diploid, codominant marker loci are supported.  Sex-linked markers are permitted. \
Support will soon extend to quantitative traits, once I hook up the disequilibrium estimator properly.  After \
that, I will add support for analyses using dominant and haploid markers."<<endl<<endl;
		}//interface
		else{	
			cout<<"Starting ClineFit..."<<endl;
			}
	if(interface){
		while(!OK){
			OK=true;
			cout<<"Type the information as it is requested below, and press \'return\' or \'enter\'."<<endl<<endl;

			bool defaultseed=false;
			cout<<"\tChoose an integer between 1 and 32666 to seed the random number generator.  Using the same seed \
twice gives identical results, useful if you want repeatability.  If you enter 0, a unique seed will be chosen for \
you and recorded in the output file."<<endl;
			cout<<"? seed = "<<endl;
			cin>>longseed;
			if(longseed <= 0){
				defaultseed=true;
				longseed = watch->first();//uses initial time as random number seed
				}
			randomNumberSeed = longseed % 32667;//mod operator

			char rawfilename[33];
			cout<<"\tWhat is the name of the data file you would like to read?  The file must be a text file, and it \
may be in my own file format or in Arlequin file format.  If in Arlequin format, it should end with the extension \
.arp (or .ARP).  Regardless, the maximum length of the name, extension and all, is 26 characters."<<endl;
			cout<<"? data file name = "<<endl;
			cin>>rawfilename;
			rawfilename[26]='\0';//truncate if overflow
			dataFileName = rawfilename;
			
			cout<<"\tPlease supply a base name for your output files.  The base name should be no more than 25 characters,\
as several files with different extensions will be generated from this base name.  Don\'t give the \
filename any extensions, and don\'t use your input file name.  BE WARNED: use a name that is unique, since all files with this same base name will \
be overwritten."<<endl;
			cout<<"? output file name = "<<endl;
			cin>>rawfilename;
			rawfilename[25]='\0';//truncate if overflow
			outputFileName = rawfilename;
			
			cout<<endl<<"The program's current settings are:"<<endl;
				cout<<"\tRandom number seed: "<<randomNumberSeed;if(defaultseed) cout<<"  (set for you)"; cout<<endl;
				cout<<"\tInput file name: "<<dataFileName<<endl;
				cout<<"\tBase output file name: "<<outputFileName<<endl;
			cout<<endl<<"Do you want to use these settings? (Y/N)"<<endl;
			cin>>yn;
			if((yn == 'Y') || (yn == 'y')){OK=true;}else {OK=false;}
			}//end while
		}//interface
	long seed = randomNumberSeed;//integer between 1 and 32667
	Ran3 ran(-seed);

	//THIS IS FOR CODOMINANT GENOTYPIC DATA 
	ClineData *dataSetUsed = NULL;
	ClineGeneticData origAlloData, *alloDataUsed = NULL;
	ClineGeneticData origmtDNAData, *mtDNADataUsed = NULL;
	ClinePhenotypicData morphData;
	aString mtdnaDatatype("haplotype");
	int numTraits = 0;
	int lociBeingFit = 3;//set to nloci in data if you toggle genotypicDataUsed above;
	int quantTraitsBeingFit = 0;//set if you toggle morphologicaDataUsed above
	aList<double> popLocation;
	StringList allPopList, traitNames;
	if (genotypicDataUsed){
			aString inFileNameCopy(dataFileName);
			aString postfixchars(".");
			aString *postfix = inFileNameCopy.ChopOffLastWord(postfixchars);
			if(*postfix=="arp" || *postfix == "ARP"){
					origAlloData.ReadArlequinDataFile(&dataFileName);//debug this
					}
				else{
					origAlloData.ReadGelJockDataFile(&dataFileName);}
			delete postfix;postfix=0;

			alloDataUsed = origAlloData.PoolDiagnosticAlleles();
			aString convertFileName(dataFileName);
			convertFileName.cat(".pool");
			alloDataUsed->PrintGelJockFile(convertFileName);
			alloDataUsed->CalculateAlleleFreqs();
			alloDataUsed->CalculateHybridIndex(popLocation);
//			CalculateHybridIndexByLocus(theDataByLocus, alloDataUsed, popLocation);
			convertFileName = dataFileName;
			convertFileName.cat(".HI");
			alloDataUsed->PrintHybridIndexFile(convertFileName);
			alloDataUsed->SetDataTypes();
			lociBeingFit = alloDataUsed->loci();
			for(pop=0;pop<alloDataUsed->npop_;++pop){
				aString pname = alloDataUsed->popName[pop];
				if(! allPopList.Includes(&pname)){
					allPopList.Append(NEW aString(pname));}
				}//pop
			for(loc=0;loc<alloDataUsed->loci();++loc){
				aString lname = alloDataUsed->locusName[loc];
				if(! traitNames.Includes(&lname)){
					traitNames.Append(NEW aString(lname));}
				}//loc
			dataSetUsed = alloDataUsed;
			}
		else if (mtDNAdataUsed){
			origmtDNAData.SetDataTypes(mtdnaDatatype);
			origmtDNAData.ReadGelJockmtDNADataFile(&dataFileName);
			mtDNADataUsed = origmtDNAData.PoolDiagnosticHaplotypes();
			aString convertFileName(dataFileName);
			convertFileName.cat(".pool");
			mtDNADataUsed->PrintGelJockFile(convertFileName);
			mtDNADataUsed->CalculateAlleleFreqs();
			mtDNADataUsed->CalculateHybridIndex(popLocation);
			convertFileName = dataFileName;
			convertFileName.cat(".HI");
			mtDNADataUsed->PrintHybridIndexFile(convertFileName);
			lociBeingFit = mtDNADataUsed->loci();
			for(pop=0;pop<mtDNADataUsed->npop_;++pop){
				aString pname = mtDNADataUsed->popName[pop];
				if(! allPopList.Includes(&pname)){
					allPopList.Append(NEW aString(pname));}
				}//pop
			for(loc=0;loc<mtDNADataUsed->loci();++loc){
				aString lname = mtDNADataUsed->locusName[loc];
				if(! traitNames.Includes(&lname)){
					traitNames.Append(NEW aString(lname));}
				}//loc
			dataSetUsed = mtDNADataUsed;
		}
		else  if (morphologicalDataUsed){
			morphData.ReadDataFile(&morphDataFileName);
			morphData.RescoreDiagnosticTraits();
			aString morphDataDumpFilename(morphDataFileName);
			morphDataDumpFilename.cat(".chk");
			morphData.PrintPhenotypeFile(morphDataDumpFilename);
			quantTraitsBeingFit = morphData.traits();
			for(pop=0;pop<morphData.npop_;++pop){
				aString pname = morphData.popName[pop];
				if(! allPopList.Includes(&pname)){
					allPopList.Append(NEW aString(pname));}
				}//pop
			for(tr=0;tr<morphData.traits();++tr){
				aString trname = morphData.traitName[tr];
				if(! traitNames.Includes(&trname)){
					traitNames.Append(NEW aString(trname));}
				}//tr
			dataSetUsed = &morphData;
		}
		else{
			return 0;
		}
	int traitsBeingFit = lociBeingFit + quantTraitsBeingFit;

	//SET NUMBER OF PARAMETERS TO ESTIMATE, AND THE CENTRAL REGION
	//DISEQUILIBRIUM WILL BE ASSESSED IN THE CENTRAL POPULATIONS ONLY
	double numChromosomes = 24;//"effective" chromosome number; USE IF FITTING CODOMINANT-MARKER DISEQUILIBRIUM
	aList<int> centralRegionPops;//THE FIRST POPULATION IS POP 0, THE SECOND IS POP 1, ETC.
	if(!interface){///set these manually
		centralRegionPops.Append(1);//ENTER EACH SEPARATELY, IN ORDER OF DATA SET
		centralRegionPops.Append(2);//These values are for the sample data; yours will be different
		centralRegionPops.Append(3);
//		centralRegionPops.Append(5);
//		centralRegionPops.Append(11);
//		centralRegionPops.Append(12);
//		centralRegionPops.Append(13);
		}
	int estimatingD=false;
	aString  delim(" ,\t");
	if(interface){
		OK=false; legal=true;
		while(!OK || !legal){
			cout<<"\tYou may fit any of several cline models to your data.  Here you choose the fundamental\
model type of up to ";
			if(traitsBeingFit>1){cout<<"9 ";} else {cout<<"8 ";}
			cout<<"parameters.  In a later step, you\'ll have the option to turn off some of these parameters \
for the whole model";
			if(traitsBeingFit>1){cout<<" or just for particular traits";}
			cout<<".  Your choices are:"<<endl;
			cout<<"\t2\tc & w -- center and width only"<<endl;
			cout<<"\t4\tc, w, pL & pR -- asymptotic frequencies on the left and right ends"<<endl;
			cout<<"\t6\tc, w, thetaL, thetaR, zL & zR -- introgression tails on either side"<<endl;
			if(traitsBeingFit>1){
				cout<<"\t7\tc, w, thetaL, thetaR, zL, zR & D -- disequilibrium in the middle"<<endl;}
			cout<<"\t8\tc, w, thetaL, thetaR, zL, zR, pL & pR"<<endl;
			if(traitsBeingFit>1){
				cout<<"\t9\tc, w, thetaL, thetaR, zL, zR, pL, pR & D"<<endl;}
			cout<<endl;
			cout<<"? base parameters = "<<endl;
			cin>>paramsToVary;
			switch(paramsToVary){
			  case 2: OK=legal=true;break;
			  case 4: OK=legal=true;break;
			  case 6: OK=legal=true;break;
			  case 7: if(traitsBeingFit>1){
			  			OK=legal=estimatingD=true;}
			  		else{legal=false;}
			  		break;
			  case 8: OK=legal=true;break;
			  case 9: if(traitsBeingFit>1){
			  			OK=legal=estimatingD=true;}
			  		else{legal=false;}
			  		break;
			  default:
			  	 legal=false;
				}
			if(!legal){
				cout<<"You chose a value that cannot be used for this data set.  Please choose again."<<endl<<endl;}

			if(legal && traitsBeingFit>1 && estimatingD){
				char rawdataline[101];rawdataline[0]='\0';
				int settingCenterPops=true, printAllPops=false, printCenterPops=false;
				while(settingCenterPops){
					cout<<"\tYou are estimating disequilibrium in the central region of the cline.  Next you have to \
manually identify the populations in this region.  There is no perfect way to do this and compromises are needed.  The model requires \
the maximum D in the middle of the cline, and D drops to zero away from the center.  The more populations used, the more precise \
the estimate but more likely it is to be downwardly biased by populations away from the center.  My rule of thumb is \
to first fit a model without disequilibrium, then identify those populations where the average frequency varies \
between roughly 20% < p < 80% of the extreme values.  In well-sampled clines, you can narrow the range further. However, in steep \
clines, you may find that you are limited in central-region populations, and may opt to include populations \
outside this range.  If you do, be prepared to interpret your results accordingly."<<endl;
					cout<<"\tIf you want to reset the number of base parameters and estimate the cline shape without \
disequilibrium, enter \'reset\'.  If you would like to see a list of the populations by number, enter \'pops\'.  \
Otherwise, enter the numbers of the populations that you want to include in the central region.  Separate \
population numbers by commas, but no white space. For example, if you have populations in your data numbered 1-15, you \
might enter \'7,8,9,10\'.  If you enter \'7, 8, 9, 10\', it will not be understood properly."<<endl;
					cout<<"? population numbers (#,#,#/reset/pops) = "<<endl;
					cin>>rawdataline;
					aString dataline(rawdataline);
					dataline.CullLeadingWhiteSpace();
					dataline.CullTrailingWhiteSpace();
					char c1=dataline[0];
					int c1int=CharToInteger(c1);
//					if(c1int<0 && c1int>9){//first character is not an integer;
					if(c1int<0 || c1int>9){//first character is not an integer;
							if(dataline =="reset" || dataline == "RESET"){
									estimatingD=OK=settingCenterPops=false;}
								else if(dataline =="pops" || dataline == "POPS"){
									printAllPops=true;}
								else{//nonsense entered
									cout<<"Didn\'t understand the command. Try again..."<<endl;}
							}//non-integer
						else{//1st character is an integer
							///strip off numbers and append them to the population list
							centralRegionPops.Clear();
							aString *w1;
							printCenterPops=true;						
							while(dataline.len()>0){
								w1 = dataline.ChopOffFirstWord(delim);
								int popnum = w1->toInt() - 1;//0-indexed in data structures
								if((popnum-1)<0 || (popnum-1)>=allPopList.len()){//it's illegal
									cout<<"There is no population number "<<popnum<<".  Please try again..."<<endl;
									printAllPops=true;
									printCenterPops=false;
									delete w1; w1=0;//clean up
									break;
									}
								centralRegionPops.Append(popnum);
								delete w1; w1=0;
								}
							}//integers entered
					if(printAllPops){
						printAllPops=false;
						cout<<"Population by number:"<<endl;
						for(pop=0;pop<allPopList.len();++pop){
							cout<<"\t"<<pop+1<<" "<<allPopList[pop]<<endl;}
						}
					if(printCenterPops){
						printCenterPops=false;
						cout<<"Populations in the center region:"<<endl;
						for(pop=0;pop<centralRegionPops.len();++pop){
							cout<<"\t"<<centralRegionPops[pop]+1<<" "<<allPopList[centralRegionPops[pop]]<<endl;}
						cout<<endl<<"Are these populations correct (Y/N)?"<<endl;
						cin>>yn;
						if(yn=='y' || yn=='Y'){
							settingCenterPops=false;//they're now set
							OK=legal=true;
							}//yn
						}//if print
					}//while settingCenterPops
				if(estimatingD){
					dataSetUsed->SetCenterPopList(centralRegionPops);
					legal=false;
					while(!legal){
						cout<<endl<<"\tSeveral parameters can be estimated from cline shape and disequilibrium provided \
	that the harmonic mean recombination rate is known.  This can be calculated from the effective number of \
	chromosomes.  The effective number is the number of chromosomes there would be if they were all the same length."<<endl;
						cout<<"\tEnter your best estimate of the effective number of chromosomes"<<endl;
						cin>>numChromosomes;
						if(numChromosomes>0.0){legal=true;}
						}//while legal
					}//estimatingD
				}//legal & estimatingD

			}//end while
		}//interface
	//IF YOU WANT TO DESIGN YOUR OWN FIT MODEL, USE THESE SWITCHES IN THE 8-9 PARAMETER MODELS
	int regionsBeingFit = 3;//default left, center, right; otherwise set to number of sample populations on cline
	ObjList<aList<int> > paramsTurnedOn;///index [trait][param]
			for(tr=0;tr<traitsBeingFit;++tr){//default all are turned on
				paramsTurnedOn.Append(NEW aList<int>());
				for(p=0;p<paramsToVary;++p){
					paramsTurnedOn[tr].Append(1);}}
	if(!interface){
		//	paramsTurnedOn[tr][0] = 0;//turn off center_ --> why ever do this?  no reason!
		//	paramsTurnedOn[tr][1] = 0;//turn off width_  --> why ever do this?  no reason!
		//	paramsTurnedOn[tr][2] = paramsTurnedOn[tr][4] = 0;//turn off thetaL_ && zL_ -- always together
		//	paramsTurnedOn[0][2] = paramsTurnedOn[0][4] = 0;//turn off thetaL_ && zL_ -- always together
		//	paramsTurnedOn[tr][3] = paramsTurnedOn[tr][5] = 0;//turn off thetaR_ && zR_ -- always together
		//	paramsTurnedOn[tr][6] = 0;   //turn off leftSideScore_ of trait tr, fixing its left side to p=0.0
		//	paramsTurnedOn[1][6] = 0; 
				//turn off left side traits first
		//	paramsTurnedOn[tr][7] = 0;//turn off rightSideScore_ fixing its right side to p=1.0
		//	paramsTurnedOn[2][7] = 0; 
				//turn off right side traits next

		//	paramsTurnedOn[tr].Last() = 0;//turn off disequilibrium estimate last; not yet used for quantitative traits
		// etc.
			}
		else{//using interface
		 if(paramsToVary>2){//can't do this with a 2-parameter model
		 	int settingOnOff=true, printOnOff=false;
		 	while(settingOnOff){
				cout<<"By default, cline shape parameters are estimated using all the traits.  Here, \
you have an opportunity to entirely omit some traits from estimation.  For example, you may wish to skip the estimation of \
introgression tails for one of your traits, instead just estimating a 2-parameter model.  Or, you may wish to skip \
the estimation of an introgression tail on (say) the left side for one of your traits.  Or, you may have some traits that \
are fixed at pL=0 or pR=1 at the ends of the cline, and others for which you need to estimate asymptotic frequencies."<<endl;
				cout<<"\t(Enter \'default\' at anytime if you wish to return to the default condition.)"<<endl;
				cout<<"Do you have any parameters that you would like to turn on/off? (Y/N/default)"<<endl;
				char rawinputline[101];rawinputline[0]='\0';
				cin>>rawinputline;
				aString inputline(rawinputline);
				inputline.CullLeadingWhiteSpace();
				inputline.CullTrailingWhiteSpace();
				if(inputline=="default"||inputline=="DEFAULT"){
						for(tr=0;tr<traitsBeingFit;++tr){//default all are turned on
							for(p=0;p<paramsToVary;++p){
								paramsTurnedOn[tr][p]=1;}}
						cout<<"You have returned to the default condition for all parameters.  All are being estimated."<<endl;
						}//if default
					else if(inputline=="y" || inputline=="Y"){
						cout<<"\tFirst you may choose a parameter, then you will get a choice of traits for that parameter.  \
Enter \'done\' when you are satisfied with your settings."<<endl;
						cout<<"The parameters you can turn on/off are:"<<endl;
						if(paramsToVary>4){
							cout<<"\ttzL\t-- thetaL and zL: introgression tail on the left side"<<endl;
							cout<<"\ttzR\t-- thetaR and zR: introgression tail on the right side"<<endl;
							cout<<"\t\t(theta and z must be turned on/off together for a given trait)"<<endl;
							}
						if(paramsToVary==4 || paramsToVary>7){
							cout<<"\tpL\t-- asymptotic allele frequency on the left side"<<endl;
							cout<<"\t\t(if turned off, you assume pL=0 asymptotically for that trait)"<<endl;
							cout<<"\tpR\t-- asymptotic allele frequency on the right side"<<endl;
							cout<<"\t\t(if turned off, you assume pR=1 asymptotically for that trait)"<<endl;
							}
						cout<<"Which do wish to turn on/off? (";
						if(paramsToVary>4){cout<<"tzL/tzR/";}
						if(paramsToVary==4 || paramsToVary>7){cout<<"pL/pR/";}
						cout<<"done/default)"<<endl;
						char rawinputline2[101];rawinputline2[0]='\0';
						cin>>rawinputline2;
						inputline=rawinputline2;
						inputline.CullLeadingWhiteSpace();
						inputline.CullTrailingWhiteSpace();
						int param, setTrait=false;
						if(inputline=="default" || inputline=="DEFAULT"){
								for(tr=0;tr<traitsBeingFit;++tr){//default all are turned on
									for(p=0;p<paramsToVary;++p){
										paramsTurnedOn[tr][p]=1;}}
								cout<<"You have returned to the default condition for all parameters.  All are being estimated."<<endl;
								}//if default
							else if(inputline=="done" || inputline=="DONE"){
								settingOnOff=false;}
							else{
								if(inputline=="tzL" || inputline=="tzl" || inputline=="TZL"){
										param=2; setTrait=true;}
									else if(inputline=="tzR" || inputline=="tzr" || inputline=="TZR"){
										param=3; setTrait=true;}
									else if(inputline=="pL" || inputline=="pl" || inputline=="PL"){
										if(paramsToVary==4) {
												param=2;}
											else{ param=6;}
										setTrait=true;}
									else if(inputline=="pR" || inputline=="pr" || inputline=="PR"){
										if(paramsToVary==4) {
												param=3;}
											else{ param=7;}
										setTrait=true;}
									else{
										setTrait=false;
										cout<<"Didn't understand the request: "<<inputline<<endl;
									}//if elseif inputline
								}//inputline...else
						while(setTrait){
							cout<<"The current settings for parameter "<<inputline<<" are:"<<endl;
							cout<<"trait#\ton/off\ttrait name"<<endl;
							aString trstr;
							for(tr=0;tr<paramsTurnedOn.len();++tr){
								trstr.clear();  trstr.cat(tr+1);
								trstr.cat('\t');
								if(paramsTurnedOn[tr][param]){
									trstr.cat("on\t\t");}
									else{trstr.cat("off\t\t");}
								trstr.cat(traitNames[tr]);
								if(!(paramsTurnedOn[tr][param])){
									if(inputline=="pL" || inputline=="pl" || inputline=="PL"){
											trstr.cat("\t(assumes pL=0.0 for this trait)");}
										else if(inputline=="pR" || inputline=="pr" || inputline=="PR"){
											trstr.cat("\t(assumes pR=1.0 for this trait)");}
										else{}
									}//if off
								cout<<'\t'<<trstr<<endl;
								}//tr
							cout<<"\tTo change a setting, enter the trait name with the state you want it \
to show in parentheses.  For example, if you want to turn off the estimation of trait 1, enter \'1(off)\'.  To \
turn back on the estimation at trait 2, you would enter \'2(on)\'.  Don\'t put any white space between the trait \
and the parentheses -- \'1 (off)\' won\'t be understood correctly.  To set all the traits back to the default \
state (on) for this parameter, enter \'default\'.  When finished with this parameter, enter 'done'."<<endl;
							cout<<"Enter the trait number and setting  [#(on)/#(off)/default/done]"<<endl;
							char rawtraitline[101];rawtraitline[0]='\0';
							cin>>rawtraitline;
							aString trline(rawtraitline), parDelim("()");
							trline.CullLeadingWhiteSpace();
							trline.CullTrailingWhiteSpace();
							trline.CullTrailingChar(')');
							if(trline=="default" || trline=="DEFAULT"){
									for(tr=0;tr<paramsTurnedOn.len();++tr){
										paramsTurnedOn[tr][param]=1;
										if(param==2 && paramsToVary>=6){
											paramsTurnedOn[tr][4]=1;}
										if(param==3 && paramsToVary>=6){
											paramsTurnedOn[tr][5]=1;}
										}//tr
									cout<<"All values reset to their default state for this parameter"<<endl<<endl;
									}//default
								else if(trline=="done" || trline=="DONE"){
									setTrait=false;}
								else{
									aString *trnumstr=trline.ChopOffFirstWord(parDelim);
									trnum = trnumstr->toInt() - 1;//0-indexed in arrays
									if(trnum<0 ||trnum >= paramsTurnedOn.len()){
										cout<<"Trait "<<trnum+1<<" doesn\'t exist, so you can\'t set it.  Please try again..."<<endl;}
										else{//valid trait number
											aString *trval = trline.ChopOffFirstWord(parDelim);//in case extra stuff tacked on by user
											if(*trval=="off" || *trval=="OFF"){
													paramsTurnedOn[trnum][param]=0;
													if(param==2 && paramsToVary>=6){
														paramsTurnedOn[trnum][4]=0;}
													if(param==3 && paramsToVary>=6){
														paramsTurnedOn[trnum][5]=0;}
													}
												else if(*trval=="on" || *trval=="ON"){
													paramsTurnedOn[trnum][param]=1;
													if(param==2 && paramsToVary>=6){
														paramsTurnedOn[trnum][4]=1;}
													if(param==3 && paramsToVary>=6){
														paramsTurnedOn[trnum][5]=1;}
													}
												else{
													cout<<"Didn't understand command "<<*trval<<".  Please try again"<<endl;
													setTrait=true;
													}
											delete trval; trval=0;
											}//else trnum
									delete trnumstr; trnumstr=0;
									}//trline
							}//setTrait
						}//turning some on/off
					else{///all done
						settingOnOff=false;
						}
				}//while settingOnOff
			  }//paramsToVary>2
			}//end if interface		


	//HERE WE SET THE PARAMETERS THAT COVARY AMONG TRAITS
	ObjList<aList<int> > paramIdentities;
	for(t=0;t<traitsBeingFit;++t){
		paramIdentities.Append(NEW aList<int>());
		for(p=0;p<paramsToVary;++p){
			paramIdentities[t].Append(0);}//default co-estimate parameters for different traits
		}//t
	int plnum=0, prnum=0;//default: if estimated, pL and pR are independent for all traits
	if(paramsToVary==4){
			plnum=2;prnum=3;}
		else if(paramsToVary>7){
			plnum=6;prnum=7;}
		else{}
	if(plnum>0){
		int plcount=0;
		for(t=0;t<traitsBeingFit;++t){
			if(paramsTurnedOn[t][plnum]){
				plcount++;
				paramIdentities[t][plnum]=plcount;}
			}//t
		}//plnum>0
	if(prnum>0){
		int prcount=0;
		for(t=0;t<traitsBeingFit;++t){
			if(paramsTurnedOn[t][prnum]){
				prcount++;
				paramIdentities[t][prnum]=prcount;}
			}//t
		}//prnum>0

	if(!interface){//SET THEM MANUALLY
//			paramIdentities[2][2]=1;
//			paramIdentities[2][4]=1;
//			paramIdentities[1][7]=1;
//			paramIdentities[2][5]=1;

			}
		else{//using interface
		  if(traitsBeingFit>1){
		 	int settingIdentities=true, printOnOff=false, setOneAlready=false;
		 	while(settingIdentities){
				cout<<endl<<"By default, common cline shape parameters are estimated using information from all \
the traits.  For example, one common center location is co-estimated for all traits.  ";
				if(plnum>0 || prnum>0){
					cout<<"(The default exception is that pL and pR are estimated separately for each trait.)  ";}
				cout<<"Here, you can allow sets of traits to vary independently.  For example, \
you may wish to estimate separate centers for each trait, or a width shared by traits 1 & 2 that differs from the \
width estimated for traits 3-5."<<endl;
				cout<<"\tFirst you will choose the parameter you want to adjust, then you can assign \
identities to control co-estimation among the traits.  Enter \'default\' at anytime if you wish to restore the \
default states.)"<<endl<<endl;
				cout<<"Do you have any ";
				if(setOneAlready){cout<<"more ";}
				cout<<"parameters that you would like to adjust? (Y/N/default)"<<endl;
				char rawinputline[101];rawinputline[0]='\0';
				cin>>rawinputline;
				aString inputline(rawinputline);
				inputline.CullLeadingWhiteSpace();
				inputline.CullTrailingWhiteSpace();
				if(inputline=="default"||inputline=="DEFAULT"){
					for(t=0;t<traitsBeingFit;++t){
						for(p=0;p<paramsToVary;++p){
							paramIdentities[t][p]=0;}//default co-estimate parameters for different traits
						}//t
					if(plnum>0){
						int plcount=0;
						for(t=0;t<traitsBeingFit;++t){
							if(paramsTurnedOn[t][plnum]){
								plcount++;
								paramIdentities[t][plnum]=plcount;}
							}//t
						}//plnum>0
					if(prnum>0){
						int prcount=0;
						for(t=0;t<traitsBeingFit;++t){
							if(paramsTurnedOn[t][prnum]){
								prcount++;
								paramIdentities[t][prnum]=prcount;}
							}//t
						}//prnum>0
						cout<<"You have returned to the default condition for all parameters."<<endl;
						}//if default
					else if(inputline=="y" || inputline=="Y"){
						setOneAlready=true;
						cout<<"\tFirst choose a parameter, then you will get a choice of traits for that parameter.  \
	Enter \'done\' when you are satisfied with your settings."<<endl;
						cout<<"The parameter co-estimation settings you can modify are:"<<endl;
						cout<<"\tc\t-- centers"<<endl;
						cout<<"\tw\t-- widths"<<endl;
						if(paramsToVary>4){
							cout<<"\ttzL\t-- thetaL and zL: introgression tail on the left side"<<endl;
							cout<<"\ttzR\t-- thetaR and zR: introgression tail on the right side"<<endl;
							cout<<"\t\t(co-estimation of theta and z is set simultaneously for a given trait)"<<endl;
							}
						if(paramsToVary==4 || paramsToVary>7){
							cout<<"\tpL\t-- asymptotic allele frequency on the left side"<<endl;
							cout<<"\tpR\t-- asymptotic allele frequency on the right side"<<endl;
							}
						cout<<"Which do wish to adjust? (";
						cout<<"c/w/";
						if(paramsToVary>4){cout<<"tzL/tzR/";}
						if(paramsToVary==4 || paramsToVary>7){cout<<"pL/pR/";}
						cout<<"default/done)"<<endl;
						cin>>rawinputline;
						inputline=rawinputline;
						inputline.CullLeadingWhiteSpace();
						inputline.CullTrailingWhiteSpace();
						int param, setTrait=false;
						if(inputline=="default" || inputline=="DEFAULT"){
								for(t=0;t<traitsBeingFit;++t){
									for(p=0;p<paramsToVary;++p){
										paramIdentities[t][p]=0;}//default co-estimate parameters for different traits
									}//t
								if(plnum>0){
									int plcount=0;
									for(t=0;t<traitsBeingFit;++t){
										if(paramsTurnedOn[t][plnum]){
											plcount++;
											paramIdentities[t][plnum]=plcount;}
										}//t
									}//plnum>0
								if(prnum>0){
									int prcount=0;
									for(t=0;t<traitsBeingFit;++t){
										if(paramsTurnedOn[t][prnum]){
											prcount++;
											paramIdentities[t][prnum]=prcount;}
										}//t
									}//prnum>0
								cout<<"You have returned to the default condition for all parameters."<<endl;
								}//if default
							else if(inputline=="done" || inputline=="DONE"){
								settingIdentities=false;}
							else{
								if(inputline=="c" || inputline=="C"){
										param=0; setTrait=true;}
									else if(inputline=="w" || inputline=="W"){
										param=1; setTrait=true;}
									else if(inputline=="tzL" || inputline=="tzl" || inputline=="TZL"){
										param=2; setTrait=true;}
									else if(inputline=="tzR" || inputline=="tzr" || inputline=="TZR"){
										param=3; setTrait=true;}
									else if(inputline=="pL" || inputline=="pl" || inputline=="PL"){
										if(paramsToVary==4) {
												param=2;}
											else{ param=6;}
										setTrait=true;}
									else if(inputline=="pR" || inputline=="pr" || inputline=="PR"){
										if(paramsToVary==4) {
												param=3;}
											else{ param=7;}
										setTrait=true;}
									else{
										setTrait=false;
										cout<<"Didn't understand the request: "<<inputline<<endl;
									}//if elseif inputline
								}//inputline...else
						while(setTrait){
							cout<<"\tTraits with the same state in the table below will be co-estimated.  Traits that \
were turned off previously are omitted from the table.  The current co-estimation settings for parameter "<<
								inputline<<" are:"<<endl;
							cout<<"trait#\tstate\ttrait name"<<endl;
							aString trstr;
							for(tr=0;tr<paramsTurnedOn.len();++tr){
								if(paramsTurnedOn[tr][param]){
									trstr.clear();  trstr.cat(tr+1);
									trstr.cat('\t');
									trstr.cat(paramIdentities[tr][param]);
									trstr.cat("\t\t");
									trstr.cat(traitNames[tr]);
									cout<<'\t'<<trstr<<endl;
									}}//if on, for tr
							cout<<"\tTo change a setting, enter the trait number with the state you want it \
to show in parentheses.  For example, if you want to estimate trait 2 separately, enter \'2(1)\', where \'1\' is a state that \
that isn\'t in the list yet.  If you then want trait 3 to be estimated with trait 2, enter \'3(1)\'.  To \
estimate all the traits separately, enter \'independent\'.  To set all the traits back to the default state for \
this parameter, enter \'default\'.  When finished with this parameter, enter \'done\'."<<endl;
							cout<<"Enter the trait number and setting  [#(state)/independent/default/done]"<<endl;
							char rawtraitline2[101];rawtraitline2[0]='\0';
							cin>>rawtraitline2;
							aString trline2(rawtraitline2), parDelim("()"), origTrLine(rawtraitline2);
							trline2.CullLeadingWhiteSpace();
							trline2.CullTrailingWhiteSpace();
							trline2.CullTrailingChar(')');
							if(trline2=="default" || trline2=="DEFAULT"){
								if(param==plnum){
										int plcount=0;
										for(t=0;t<traitsBeingFit;++t){
											if(paramsTurnedOn[t][param]){
												plcount++;
												paramIdentities[t][param]=plcount;}
											}//t
										}//plnum
									else if(param==prnum){
										int prcount=0;
										for(t=0;t<traitsBeingFit;++t){
											if(paramsTurnedOn[t][prnum]){
												prcount++;
												paramIdentities[t][prnum]=prcount;}
											}//t
										}//prnum>0
									else{//not pL or pR
										for(t=0;t<traitsBeingFit;++t){
											paramIdentities[t][param]=0;}//default co-estimate parameters for different traits
										}
									cout<<"All values reset to their default state for this parameter"<<endl<<endl;
									}//default
								else if(trline2=="independent" || trline2=="INDEPENDENT"){
										int pcount=0;
										for(t=0;t<traitsBeingFit;++t){
											if(paramsTurnedOn[t][param]){
												paramIdentities[t][param]=pcount;
												pcount++;}
											}//t
										cout<<"This parameter will be estimated separately for each trait."<<endl<<endl;
										}//independent
								else if(trline2=="done" || trline2=="DONE"){
									setTrait=false;}
								else{
									aString *trnumstr=trline2.ChopOffFirstWord(parDelim);
									if(!(trnumstr->isNumber())){
											cout<<"Didn't understand command "<<origTrLine<<".  Please try again"<<endl;}
										else{
											trnum = trnumstr->toInt() - 1;//0-indexed in arrays
											if(trnum<0 ||trnum >= paramIdentities.len()){
													cout<<"Trait "<<trnum+1<<" doesn\'t exist, so you can\'t set it.  Please try again..."<<endl;}
												else if(paramsTurnedOn[trnum][param]==0){
													cout<<"Trait "<<trnum+1<<" isn\'t being estimated, so you can\'t set it.  Please try again..."<<endl;}										
												else{//valid trait number
													aString *trval = trline2.ChopOffFirstWord(parDelim);//in case extra stuff tacked on by user
													if(trval->isNumber()){
															int setval = trval->toInt();
															paramIdentities[trnum][param]=setval;
															if(param==2 && paramsToVary>=6){
																paramIdentities[trnum][4]=setval;}
															if(param==3 && paramsToVary>=6){
																	paramIdentities[trnum][5]=setval;}
															}
														else{//isn't a number
															cout<<"Didn't understand command "<<origTrLine<<".  Please try again"<<endl;
															}
													delete trval; trval=0;
													}//else trnum
											}//isNumber
									delete trnumstr; trnumstr=0;
									}//trline2
							}//setTrait
						}//settingIdentities
					else{///all done
						settingIdentities=false;
						}
				}//while settingOnOff
			  }//traitsBeingFit>1
			}//end if interface		
	//need to sort paramIdentities from low to high
	int varyingparams=0;//a counter of parameters that will actually vary in the estimation
	ObjList<aList<int> > uniqueParamIdentities;//list of unique settings for each parameter
	for(int pa=0;pa<paramsToVary;++pa){
		uniqueParamIdentities.Append(NEW aList<int>());
			for(tr=0;tr<paramIdentities.len();++tr){
				if(! (uniqueParamIdentities[pa].Includes(paramIdentities[tr][pa]))){
					uniqueParamIdentities[pa].Append(paramIdentities[tr][pa]);
					varyingparams++;
					}
			}
		}
	//use these settings to rescore paramIdentities so that first is 0, next is 1, etc for each parameter
	for(tr=0;tr<paramIdentities.len();++tr){
		for(int pa=0;pa<paramIdentities[tr].len();++pa){
			int val = uniqueParamIdentities[pa].Loc(paramIdentities[tr][pa]);//get index 
			paramIdentities[tr][pa]=val;//set to index
			}
		}
			
	double meanRecomb = 0.5;	//mean recombination rate among allozyme loci
	double leftEnd;
	double rightEnd;
	leftEnd = rightEnd = dataSetUsed->Locality(0);
	for(pop=1;pop<dataSetUsed->pops();++pop){
		double thisLocality = dataSetUsed->Locality(pop);
		leftEnd = MIN(thisLocality,leftEnd);
		rightEnd = MAX(thisLocality,rightEnd);
		}//end for pop


	if(interface){//determine what the analysis will do
		OK = false;
		while(! OK){
			cout<<"\tYou may run this program in order to estimate *new* cline shape parameters that fit your data best, \
or else you may *test* the fit of your data to a set of parameter values that you supply."<<endl;
			cout<<"Which do you wish to do? [new/test]"<<endl;
			char testfitlineraw[101];testfitlineraw[0]='\0';
			cin>>testfitlineraw;
			testfitlineraw[100]='\0';
			aString testfitline(testfitlineraw);
			testfitline.CullLeadingWhiteSpace();
			testfitline.CullTrailingWhiteSpace();
			if(testfitline == "new" || testfitline == "NEW"){
					testingFitToGivenShape=false; OK=true;}
				else if(testfitline == "test" || testfitline == "TEST"){
					testingFitToGivenShape=true; OK=true;}
				else{
					cout<<"Didn\'t understand the command \'"<<testfitline<<"\'."<<endl;
					cout<<"Please respond with \'new\' or \'test\'."<<endl<<endl;
					OK=false;}
			}//OK
		OK = testingFitToGivenShape;
		while(! OK){
			cout<<"\tYou may initialize this run with *precise* values that you supply for all parameters (perhaps from prior information \
or a previous run), or you may start from rough settings of only the most critical parameters."<<endl;
			cout<<"Which do you wish to do? [precise/rough]"<<endl;
			char initfitlineraw[101];initfitlineraw[0]='\0';
			cin>>initfitlineraw;
			initfitlineraw[100]='\0';
			aString initfitline(initfitlineraw);
			initfitline.CullLeadingWhiteSpace();
			initfitline.CullTrailingWhiteSpace();
			if(initfitline == "precise" || initfitline == "PRECISE"){
					enteringStartingValues=true; OK=true;}
				else if(initfitline == "rough" || initfitline == "ROUGH"){
					enteringStartingValues=false; OK=true;}
				else{
					cout<<"Didn\'t understand the command \'"<<initfitline<<"\'."<<endl;
					cout<<"Please respond with \'precise\' or \'rough\'."<<endl<<endl;
					OK=false;}
			}//OK
		}//interface



	//ENTER SOME INITIAL PARAMETER ESTIMATES
	long double Icenter = leftEnd + (rightEnd - leftEnd)/2.0;//15.0 for sample data; your values will be different
	long double Iw = ABS(0.20 * (rightEnd - leftEnd));//2.0 
	long double IthetaL = 0.1;//0.05 
	long double IthetaR = 0.1;//0.05 
	long double IzL = Icenter - 1.5*Iw;//2.5; make these wider than initial width
	long double IzR = Icenter + 1.5*Iw;//2.5
	long double Idiseq = 0.0;
	aList<long double> ImeanLowScore;
	aList<long double> ImeanHighScore;
	for(loc=0;loc<lociBeingFit;++loc){
		ImeanLowScore.Append(0.0);//one for each trait/locus in your data (must enter)
		ImeanHighScore.Append(1.0);//one for each trait/locus in your data (must enter)
		}
	for(tr=0;tr<quantTraitsBeingFit;++tr){
		ImeanLowScore.Append(-100.0);//one for each trait/locus in your data (must enter)
		ImeanHighScore.Append(100.0);//one for each trait/locus in your data (must enter)
		}
	MatrixList Icov,IcovOnOff;//covariance matrices for morphological traits
	for(r=0;r<regionsBeingFit;++r){//create empty matrix; fill it manually
		Icov.Append(IdentityMatrix((long double) (0.0),quantTraitsBeingFit));
		Icov[r].Multiply(2.0);// this is the initial variance, same for all traits
		IcovOnOff.Append(OneMatrix((long double) (1.0),quantTraitsBeingFit,quantTraitsBeingFit));//all assumed on at start
		}//r
//	Icov[region][0][0]  = 0.5;//use this format to set particular variances
//	Icov[region][0][1] = Icov[region][1][0] = 0.5;//use this symmetrical format to set particular covariances
//	IcovOnOff[region][0][1] = IcovOnOff[region][1][0] = 0;//use this symmetrical format to toggle particular covariances
//	IcovOnOff[region][0][0] = 0;//use this symmetrical format to toggle particular variances

	if(interface && (! (testingFitToGivenShape || enteringStartingValues) ) ){
		cout<<"\tNext you need to supply starting values for the center";
		if(paramsToVary==4 || paramsToVary>7){
			cout<<", width, and the left and right sides of the cline for each trait";}
			else{cout<<" and width";}
			cout<<".  Numerical estimation runs more efficiently if your starting shape roughly \
approximates the shape of the cline.  Other starting parameters are less important and are chosen automatically.";
		if(traitsBeingFit>1){
				cout<<"  Even if you are estimating multiple centers or widths, just enter your guess at \
the average value.";}
		int startingEntered=false;
		while(!startingEntered){
			cout<<"  Enter \'c\' or \'w\', ";
			if(paramsToVary==4 || paramsToVary>7){
					cout<<"\'pL\', \'pR\', ";}
			cout<<"or \'done\' if you are finished"<<endl;
			cout<<"\t\tc:\t"<<Icenter<<endl;
			cout<<"\t\tw:\t"<<Iw<<endl;
			int leftOn=false,rightOn=false;
			int pp=0;  if(paramsToVary==4){pp=2;}else if(paramsToVary>7){pp=6;}else{}//pL parameter
			if(pp>0){
				for(tr0=0;tr0<traitsBeingFit;++tr0){
					if(paramsTurnedOn[tr0][pp]){leftOn=true;}
					if(paramsTurnedOn[tr0][pp+1]){rightOn=true;}}
				for(int tr1=0;tr1<traitsBeingFit;++tr1){
					int alreadyprinted=false;
					aString thisline("\t\tpL[");
					if(leftOn && paramsTurnedOn[tr1][pp]){
							int sharedState=paramIdentities[tr1][pp];
							for(int tr2=0;tr2<traitsBeingFit;++tr2){
								if(paramsTurnedOn[tr2][pp]){
									if(paramIdentities[tr2][pp]==sharedState){
										if(tr2<tr1){alreadyprinted=true;}
										if(tr2>tr1){thisline.cat(";");}
										thisline.cat(tr2+1);}}}
							thisline.cat("]\t");thisline.cat(ImeanLowScore[tr1]);thisline.cat("\t\t");
							if(!alreadyprinted){cout<<thisline;}
							}///leftOn
						else{cout<<"\t\t\t\t\t";}
					alreadyprinted=false;thisline="\t\tpR[";
					if(rightOn && paramsTurnedOn[tr1][pp+1]){
						int sharedState=paramIdentities[tr1][pp+1];
						for(int tr3=0;tr3<traitsBeingFit;++tr3){
							if(paramsTurnedOn[tr3][pp+1]){
								if(paramIdentities[tr3][pp+1]==sharedState){
									if(tr3<tr1){alreadyprinted=true;}
									if(tr3>tr1){thisline.cat(";");}
									thisline.cat(tr3+1);}}}
						thisline.cat("]\t");thisline.cat(ImeanHighScore[tr1]);
						if(!alreadyprinted){cout<<thisline;}
						}
					cout<<endl;
					}//tr1
				}//pp
			cout<<"Parameter? (c/w";
			if(leftOn){cout<<"/pL";}
			if(rightOn){cout<<"/pR";}
			cout<<"/done)"<<endl;
			char rawsettingline[21];rawsettingline[0]='\0';
			cin>>rawsettingline;
			aString settingline(rawsettingline);
			settingline.CullLeadingWhiteSpace();
			settingline.CullTrailingWhiteSpace();
			if(settingline=="done" || settingline=="DONE"){
					startingEntered=true;}
				else if(settingline=="c" || settingline=="C"){
					cout<<"center ="<<endl;
					cin>>Icenter;
					}
				else if(settingline=="w" || settingline=="W"){
					cout<<"width ="<<endl;
					cin>>Iw;
					}
				else if(settingline=="pL" || settingline=="pl" || settingline=="PL"){
					cout<<"pL for which trait?  trait="<<endl;
					int pLtr;
					cin>>pLtr;
					if(pLtr<1 || pLtr>traitsBeingFit){
						cout<<"illegal value"<<endl;}
						else{
							cout<<"pL["<<pLtr<<"]="<<endl;
							double pLval;
							cin>>pLval;
							if(pLval<0.0 || pLval>1.0){
								cout<<"illegal value"<<endl;}
								else{ImeanLowScore[pLtr-1]=pLval;}
							}//legal trait
					}//pL
				else if(settingline=="pR" || settingline=="pr" || settingline=="PR"){
					cout<<"pR for which trait?  trait="<<endl;
					int pRtr;
					cin>>pRtr;
					if(pRtr<1 || pRtr>traitsBeingFit){
						cout<<"illegal value"<<endl;}
						else{
							cout<<"pR["<<pRtr<<"]="<<endl;
							double pRval;
							cin>>pRval;
							if(pRval<0.0 || pRval>1.0){
								cout<<"illegal value"<<endl;}
								else{ImeanHighScore[pRtr-1]=pRval;}
							}//legal trait
					}//pR
				else{
					cout<<"Didn\'t understand command "<<settingline<<"  Please try again..."<<endl;}
			}//while !startingEntered
		IzL = IzR = Iw*1.5;
		}//interface


	//THESE DEFINE THE UPPER AND LOWER BOUNDS OF PARAMETER VALUES
	long double centerMin;//default = leftEnd;	
	long double centerMax;//default = rightEnd;
	long double wMin = 0.0;
	long double wMax;//default = 2.0 * ABS(rightEnd - leftEnd);	//2 x length of the transect
	long double thetaMin = 0.0;
	long double thetaMax = 1.0;
	long double zMin = 0.0;
	long double zMax = 1000.0;
	long double xLRMin;//default = leftEnd;
	long double xLRMax;//default = rightEnd;
	long double diseqMin = -0.25;
	long double diseqMax = 0.25;
	aList<long double> meanLowScoreMin;
	aList<long double> meanHighScoreMax;
	for(loc=0;loc<lociBeingFit;++loc){
		meanLowScoreMin.Append(0.0);//one for each trait/locus in your data (must enter)
		meanHighScoreMax.Append(1.0);//one for each trait/locus in your data (must enter)
		}
	aList<double> traitVarMin, traitVarMax;//for fitting quantitative traits
	for(tr=0;tr<quantTraitsBeingFit;++tr){
		meanLowScoreMin.Append(-100.0);//one for each trait/locus in your data (must enter)
		meanHighScoreMax.Append(100.0);//one for each trait/locus in your data (must enter)
		traitVarMin.Append(0.0);
		traitVarMax.Append(1.0e100);
		}
	MatrixList maxCovList, minCovList;
	for(r=0;r<regionsBeingFit;++r){
		maxCovList.Append(OneMatrix((long double) (1.0),quantTraitsBeingFit,quantTraitsBeingFit));//all deviate the same
		maxCovList[r].Multiply(1.0e100);
		minCovList.Append(OneMatrix((long double) (1.0),quantTraitsBeingFit,quantTraitsBeingFit));//all deviate the same
		minCovList[r].Multiply(-1.0e100);
		for(tr=0;tr<quantTraitsBeingFit;++tr){
			minCovList[r][tr][tr]=0.0;}//min variance =0
		}//r



	//FIDDLE WITH THESE DEVIATIONS TO GET AN EFFICIENT ANNEALING RUN
			//this is now automatically adjusted so it's unimportant to specify these
	long double centerDev = 0.25;//how much to deviate the random number by for this parameter
	long double wDev = 0.5;
	long double thetaDev = 0.02;
	long double zDev = 0.2;	//moves the barriers right or left
	long double bottomTopDev = 0.05;	//moves min & max averages up or down for quantitative traits
	double varianceDev = 0.5;	//alters phenotypic variance for quantitative traits
	long double diseqDev = 0.10;
	
	aList<long double> bottomTopDevList;
	for(t=0;t<ImeanLowScore.len();++t){
		bottomTopDevList.Append(bottomTopDev);}
	aList<double>varianceDevList;
	for(t=0;t<ImeanLowScore.len();++t){
		varianceDevList.Append(varianceDev);}
	MatrixList devCovList;
	for(r=0;r<regionsBeingFit;++r){
		devCovList.Append(OneMatrix((long double) (1.0),quantTraitsBeingFit,quantTraitsBeingFit));//all deviate the same
		devCovList[r].Multiply(varianceDev);
		}//r



	//set default parameter values -- hardwired here but needs to be set by user
	centerMin = xLRMin = leftEnd;
	centerMax = xLRMax = rightEnd;
	wMax = 2.0 * ABS(rightEnd - leftEnd);	//2 x length of the transect
	if((genotypicDataUsed || mtDNAdataUsed) &&
		((paramsToVary == 4)||(paramsToVary == 8)||(paramsToVary == 9)) &&
		(ImeanLowScore.len()<dataSetUsed->NumTraits())){
		//ensure enough copies of max & min scores for each locus
		long double lowscore = ImeanHighScore[0];
		long double highscore = ImeanHighScore[0];
		long double lowscoreMin = meanLowScoreMin[0];
		long double highscoreMax = meanHighScoreMax[0];
		for(tr=ImeanLowScore.len();tr<dataSetUsed->NumTraits();++tr){
			ImeanLowScore.Append(lowscore);
			ImeanHighScore.Append(highscore);
			meanLowScoreMin.Append(lowscoreMin);
			meanHighScoreMax.Append(highscoreMax);
			bottomTopDevList.Append(bottomTopDev);
			varianceDevList.Append(varianceDev);
		}	}//end for tr && end multiple if
	ObjList<aList<long double> > paramMin, paramMax;//index [param][trait]
	for(p=0;p<paramsToVary;++p){
		paramMin.Append(NEW aList<long double>());
		paramMax.Append(NEW aList<long double>());}
	for(tr=0;tr<traitsBeingFit;++tr){
		paramMin[0].Append(centerMin);
		paramMax[0].Append(centerMax);
		paramMin[1].Append((long double)0.0);
		paramMax[1].Append(wMax);
		if(paramsToVary == 4){
			paramMin[2].Append((long double) 0.0);//pL		
			paramMax[2].Append((long double)1.0);//pL		
			paramMin[3].Append((long double)0.0);//pR		
			paramMax[3].Append((long double)1.0);//pR	
			}					
		if(paramsToVary >= 6){
			paramMin[2].Append((long double)0.0);//thetaL		
			paramMax[2].Append((long double)1.0);//thetaL		
			paramMin[3].Append((long double)0.0);//thetaR		
			paramMax[3].Append((long double)1.0);//thetaR	
			paramMin[4].Append((long double)0.0);//zL		
			paramMax[4].Append((long double)(1.0e10));//zL	
			paramMin[5].Append((long double)0.0);//zR	
			paramMax[5].Append((long double)1.0e10);//zR
			}					
		if(paramsToVary > 7){
			paramMin[6].Append((long double)0.0);//pL		
			paramMax[6].Append((long double)1.0);//pL		
			paramMin[7].Append((long double)0.0);//pR		
			paramMax[7].Append((long double)1.0);//pR	
			}					
		}//tr



	///this is not implemented for the new on/off switches yet -- all info ignored
	//IF TESTING FIT TO A KNOWN SHAPE, ENTER THOSE PARAMETERS HERE
		//enter only relevant ones and the others will be ignored
	aList<long double> testCenter,testWidth,testThetaL,testThetaR,testZL,testZR,testMeanLowScore,testMeanHighScore;
	for(int tr3=0;tr3<traitsBeingFit;++tr3){//default values
		testCenter.Append((long double)((centerMax - centerMin)/2.0));
		testWidth.Append((long double)2.0);
		testThetaL.Append((long double)0.0);
		testThetaR.Append((long double)0.0);
		testZL.Append((long double)0.0);
		testZR.Append((long double)0.0);
		testMeanLowScore.Append((long double)0.0);
		testMeanHighScore.Append((long double)1.0);
		}

	MatrixList testCov;
	for(r=0;r<regionsBeingFit;++r){//create empty matrix; fill it manually
		testCov.Append(ZeroMatrix((long double) (0.0),quantTraitsBeingFit,quantTraitsBeingFit));
		}//r
//	testCov[region][0][0]  = 0.5;//use this format to set particular variances
//	testCov[region][0][1] = testCov[region][1][0] = 0.5;//use this symmetrical format to set particular covariances
	long double testDiseq = 		0.000000000;

	if(interface && (testingFitToGivenShape || enteringStartingValues)){
		StringList paramnames;
		aList<int> paramEstimated;//index [param]
		ObjList<aList<int> > valAssignedTF;//index [param][trait]
		ObjList<aList<long double> > valAssigned;//index [param][trait]
		ObjList<StringList> valAssignedStr;//index [param][trait]
		int testDiseqSpecified = false;
		paramnames.Append(NEW aString("c"));			
		paramnames.Append(NEW aString("w"));
		if(paramsToVary >= 6){
			paramnames.Append(NEW aString("thetaL"));			
			paramnames.Append(NEW aString("thetaR"));
			paramnames.Append(NEW aString("zL"));			
			paramnames.Append(NEW aString("zR"));
			}					
		if(paramsToVary == 4 || paramsToVary > 7){
			paramnames.Append(NEW aString("pL"));			
			paramnames.Append(NEW aString("pR"));
			}					
		if(paramsToVary == 3 || paramsToVary == 5 || paramsToVary == 7 || paramsToVary == 9){
			paramnames.Append(NEW aString("D"));			
			}					
		aString paramlist("[");
		for(p=0;p<paramsToVary;++p){
			paramlist.cat(paramnames[p]);paramlist.cat('/');
			int allOff=true;
			paramEstimated.Append(true);
			valAssignedTF.Append(NEW aList<int>());
			valAssigned.Append(NEW aList<long double>());
			valAssignedStr.Append(NEW StringList());
			for(tr=0;tr<traitsBeingFit;++tr){
				if(allOff && paramsTurnedOn[tr][p] == true){
					allOff=false;}
				valAssignedTF[p].Append(false);
				valAssigned[p].Append(0.0);
				valAssignedStr[p].Append(NEW aString());//blank
				}//tr
			if(allOff){
				paramEstimated[p]=false;}
			}//p
		paramlist.cat("done]");
		if(testingFitToGivenShape){
				cout<<"\tYou have chosen to fit your data to a given set of parameter values.  ";}
			else{
				cout<<"You have chosen to supply starting values for all parameters.  ";}
		cout<<"In this step you supply those values.  The likelihood is often very sensitive to small differences \
in parameter values, so enter as many decimal places as you are able."<<endl;

		cout<<"\tFirst you will choose the parameter type you want to assign, then you can assign its \
value for each of the traits (if there are more than one).  Enter \'done\' when you are satisfied \
with your settings."<<endl<<endl;
		OK=false;
		while(!OK){
			int tryToExit=false;
			cout<<"Current values are:"<<endl;
			for(p=0;p<paramnames.len();++p){
				if(! paramEstimated[p]) continue;//skip this one
				cout<<"\t"<<paramnames[p]<<": ";
				if(valAssignedTF[p].len()==1){
						//don't specify traits in output
						if(valAssignedTF[p][0]){
								cout<<valAssignedStr[p][0]<<endl;}
							else{
								cout<<"unspecified"<<endl;}
						}
					else{//specify traits in output
						//handle shared parameter states
						listedSoFar=0,currentState=0, firstTrait;
				//		int listedSoFar=0,firstTrait,currentStateCounter=0;
				//		aList<int> assignedSettingClasses;//series of states may have gaps
				//		for(tr=0;tr<valAssigned[p].len();++tr){
				//			if(! assignedSettingClasses.Includes(valAssigned[p][tr])){
				//				assignedSettingClasses.Append(valAssigned[p][tr]);}
				//			}
						lineout="";
						cout<<endl;
						//need to identify traits not estimated (=treated as constants)
						traitsToSkip.Clear();
						for(tr=0;tr<traitsBeingFit;++tr){//skipped traits can be scattered through the list
								pto=paramsTurnedOn[tr][p];
								if(!pto){
									traitsToSkip.Append(tr);
									listedSoFar++;
									}
								}//tr
						while(listedSoFar<valAssignedTF[p].len()){
							aList<int> traitsToList;
							for(tr=0;tr<traitsBeingFit;++tr){//skipped traits can be scattered through the list
								pto=paramsTurnedOn[tr][p],pid=paramIdentities[tr][p];
								if(pto && pid == currentState){
					//			if(pto && pid == assignedSettingClasses[currentStateCounter]){
					//			if(paramsTurnedOn[tr][p] && paramIdentities[tr][p] == currentState){
									traitsToList.Append(tr);
									listedSoFar++;}
								}//tr
							//listedSoFar += traitsToList.len();
							++currentState;
							if(traitsToList.len()>0){
								lineout="\t\t[";
								for(tr=0;tr<traitsToList.len();++tr){
									lineout.cat(traitsToList[tr]+1);
									if(tr<traitsToList.len()-1){
										lineout.cat(",");}
									}//tr
								lineout.cat("]: ");
								cout<<lineout;
								firstTrait = traitsToList[0];
								if(valAssignedTF[p][firstTrait]){
										cout<<valAssignedStr[p][firstTrait]<<endl;}
									else{
										cout<<"unspecified"<<endl;}
								}//traitstoList
							}//while
						if(traitsToSkip.len()>0){//list these at the end
							lineout="\t\t[";
							for(tr=0;tr<traitsToSkip.len();++tr){
								lineout.cat(traitsToSkip[tr]+1);
								if(tr<traitsToSkip.len()-1){
									lineout.cat(",");}
								}//tr
							lineout.cat("]: ");
							cout<<lineout;
							firstTrait = traitsToSkip[0];
							cout<<"not estimated: held constant at default value"<<endl;
							}//if traitsToSkip
							
						}//else
					}//p
			char para[101]; para[0]='\0'; 
			int goodParam=false, changeD;
			int thisparam = -1;
			while(! goodParam){
				cout<<"Choose a parameter that you would like to set. "<<paramlist<<endl;
				cout<<"parameter = ?"<<endl;
				cin>>para; para[100]='\0';
				aString paraname(para);
				paraname.CullLeadingSpaces(); paraname.CullTrailingSpaces();
				aString paramword;
				goodParam=true;
				changeD = false;
				if(paraname == "c" || paraname == "C"){paramword="c";}
					else if(paraname == "w" || paraname == "W"){ paramword="w";}
					else if(paraname == "thetaL" || paraname == "THETAL"|| paraname == "thetal"){ paramword="thetaL";}
					else if(paraname == "thetaR" || paraname == "THETAR"|| paraname == "thetar"){ paramword="thetaR";}
					else if(paraname == "zL" || paraname == "ZL"|| paraname == "zl"){ paramword="zL";}
					else if(paraname == "zR" || paraname == "ZR"|| paraname == "zr"){ paramword="zR";}
					else if(paraname == "pL" || paraname == "PL"|| paraname == "pl"){ paramword="pL";}
					else if(paraname == "pR" || paraname == "PR"|| paraname == "pr"){ paramword="pR";}
					else if(paraname == "D" || paraname == "d"){ paramword="D"; changeD = true;}
					else if(paraname == "done" || paraname == "DONE"){
						tryToExit=true;
						}
					else{goodParam=false;
						cout<<"Didn'\t recognize parameter "<<paraname<<endl;}
				if(paramnames.Includes(&paramword)){
						thisparam = paramnames.Loc(&paramword);}
					else if (! tryToExit){goodParam=false;
						cout<<"The parameter "<<paraname<<" isn\'t being estimated."<<endl;}
					else{}
				}//while goodParam
			int satisfied=false;
			if(tryToExit){
					int allTrue=true;
					for(i=0;i<valAssignedTF.len();++i){
						for(j=0;j<valAssignedTF[i].len();++j){
							if(paramsTurnedOn[j][i] && valAssignedTF[i][j]==false){
								allTrue=false;break;}}}
					if(allTrue) {OK = satisfied = true;}
						else{
							cout<<"There are still unset parameters."<<endl;satisfied=true;}
					}//try to exit
				else if(changeD){
					int Dsatisfied = false;
					while (! Dsatisfied){
						cout<<"Current value is:  D = ";
						if(testDiseqSpecified){
								cout<<testDiseq<<endl;}
							else{ cout<<"unspecified"<<endl;}
						cout<<"new D = ?"<<endl;
						cin>>testDiseq;
						//see if the new value is legal
						if(testDiseq<diseqMin || testDiseq>diseqMax){
								cout<<testDiseq<<" is out of range. Choose "<<diseqMin<<"  D  "<<diseqMax<<endl;}
							else{ Dsatisfied=testDiseqSpecified=true;}				
						}//while Dsatisfied
					satisfied=true;
					}
				else{}
			while (!satisfied){
				if(valAssignedStr[thisparam].len()>1){
						cout<<"Current values are:"<<endl;}
					else{
						cout<<"Current value is:"<<endl;}					
				int chosenTrait=0;
				ObjList<aList<int> > traitsToListBySet;
				if(valAssignedTF[thisparam].len()==1){
						//don't specify traits in output
						if(valAssignedTF[thisparam][0]){
								cout<<valAssignedStr[thisparam][0]<<endl;}
							else{
								cout<<"unspecified"<<endl;}
						traitsToListBySet.Append(NEW aList<int>());
						traitsToListBySet[0].Append(0);
						}
					else{//specify traits in output
						//handle shared parameter states
						listedSoFar=0, currentState=0;
						//need to identify traits not estimated (=treated as constants)
						traitsToSkip.Clear();
						for(tr=0;tr<traitsBeingFit;++tr){//skipped traits can be scattered through the list
								pto=paramsTurnedOn[tr][thisparam];
								if(!pto){
									traitsToSkip.Append(tr);
									listedSoFar++;
									}
								}//tr
						while(listedSoFar<valAssignedTF[thisparam].len()){
							traitsToListBySet.Append(NEW aList<int>());
							for(tr=0;tr<traitsBeingFit;++tr){
								pto=paramsTurnedOn[tr][thisparam], pid=paramIdentities[tr][thisparam];
								if(pto && pid == currentState){
									traitsToListBySet[currentState].Append(tr);
									listedSoFar++;}
								}//tr
							//listedSoFar += traitsToListBySet[currentState].len();
							if(traitsToListBySet[currentState].len()>0){
								lineout="\t\t[";
								for(tr=0;tr<traitsToListBySet[currentState].len();++tr){
									lineout.cat(traitsToListBySet[currentState][tr]+1);
									if(tr<traitsToListBySet[currentState].len()-1){
										lineout.cat(",");}
									}//tr
								lineout.cat("]: ");
								cout<<lineout;
								int firstTrait = traitsToListBySet[currentState][0];
								if(valAssignedTF[thisparam][firstTrait]){
										cout<<valAssignedStr[thisparam][firstTrait]<<endl;}
									else{
										cout<<"unspecified"<<endl;}
								}//traitsToListBySet
							++currentState;
							}//while
						if(traitsToSkip.len()>0){//list these at the end
							lineout="\t\t[";
							for(tr=0;tr<traitsToSkip.len();++tr){
								lineout.cat(traitsToSkip[tr]+1);
								if(tr<traitsToSkip.len()-1){
									lineout.cat(",");}
								}//tr
							lineout.cat("]: ");
							cout<<lineout;
							firstTrait = traitsToSkip[0];
							cout<<"not estimated: held constant at default value"<<endl;
							}//if traitsToSkip

						cout<<"Choose a trait that you would like to set a new value for."<<endl;
						cout<<"If you are satisfied with these settings, type \'done\'."<<endl;
						cin>>para;  para[100]='\0';
						aString trname(para);
						para[0]='/0';
						trname.CullLeadingSpaces(); trname.CullTrailingSpaces();
						if(trname == "done" || trname == "DONE"){
								int legalToLeave = true;
								for(tr0=0;tr0<valAssignedTF[thisparam].len();++tr0){
									if(! traitsToSkip.Includes(tr0)){
										if(valAssignedTF[thisparam][tr0] == false){
											legalToLeave = false; break;}}
									}
								if(legalToLeave){
									satisfied=true;
									}
								}
							else{
								if(! trname.isNumber()) {
										cout<<"Didn\'t understand command \'"<<trname<<"\'."<<endl;}
									else{
										chosenTrait=trname.toInt(); --chosenTrait;}
								}
						}//else choosing among >1 traits
				if(!satisfied){
					if(chosenTrait<0 || chosenTrait >= valAssignedTF[thisparam].len()){
							cout<<"Trait "<<chosenTrait+1<<" is out of range."<<endl;}
						else{
							int satisfiedBoundaries=false;
							while(! satisfiedBoundaries){
								cout<<"Please assign a new value."<<endl;
								cout<<paramnames[thisparam];
								if(valAssignedStr[thisparam].len()>1){
									cout<<"["<<chosenTrait+1<<"]";}
								cout<<" = ?"<<endl;
								char newvalword[51];newvalword[0]='\0';
								cin>>newvalword;
								newvalword[50]='\0';
								aString newvalstr(newvalword);
								long double newval=newvalstr.toLongDouble();
								//see if the new value is legal
								if(newval< paramMin[thisparam][chosenTrait] || newval > paramMax[thisparam][chosenTrait]){
										cout<<newval<<" is out of range. Choose "<<paramMin[thisparam][chosenTrait]<<"  x  "<<paramMax[thisparam][chosenTrait]<<endl;}
									else{satisfiedBoundaries=true;}
								//reset all parameters in this set
								for(int set0=0;set0<traitsToListBySet.len();++set0){
									if(traitsToListBySet[set0].Includes(chosenTrait)){
										for(int trInSet=0;trInSet<traitsToListBySet[set0].len();++trInSet){
											int thistr=traitsToListBySet[set0][trInSet];
							//				aString valStr(newval,0,20);
							//				valAssignedStr[thisparam][thistr]=valStr;
											valAssignedStr[thisparam][thistr]=newvalstr;
											valAssigned[thisparam][thistr]=newval;
											valAssignedTF[thisparam][thistr]=true;
											}//trInSet
										}//if
									}//set0
								}//while ! satisfiedBoundaries
						}//else trait in range
					}//!satisfied
				int allSet=true;
				for(int tr7=0;tr7<valAssignedTF[thisparam].len();++tr7){
					if(valAssignedTF[thisparam][tr7]==false){ allSet=false; break;}}
				if(allSet){satisfied=true;}
				
				}//satisfied
			}//OK
		for(int tr4=0;tr4<traitsBeingFit;++tr4){
			int paramLoc=-1;
			aString paramstr = "c";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testCenter[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "w";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testWidth[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "thetaL";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testThetaL[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "thetaR";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testThetaR[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "zL";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testZL[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "zR";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testZR[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "pL";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testMeanLowScore[tr4] = valAssigned[paramLoc][tr4];}
			paramstr = "pR";
			if(paramnames.Includes(&paramstr)){
				paramLoc = paramnames.Loc(&paramstr);
				testMeanHighScore[tr4] = valAssigned[paramLoc][tr4];}
			}//tr4
			


		}//interface && testingFitToGivenShape

	//*******END OF PARAMETERS THAT NEED SETTINGS


	double r0 = 10e-5;//minimum distance between loci on a chromosome
	double harMeanRecomb = HarmonicDiseqMarkerVsSelLoci(numChromosomes);
	
	int covMatrixListType = (regionsBeingFit == 3);
	
	ClineFitInitializer startingParamInitializer(leftEnd,rightEnd,
							Icenter,Iw,IthetaL,IthetaR,IzL,IzR,
  							Idiseq,harMeanRecomb,ImeanLowScore,ImeanHighScore,
  							covMatrixListType,Icov,IcovOnOff,
//  							IleftVariance,IcenterVariance,IrightVariance,
  							paramsTurnedOn,paramIdentities,dataFileName,outputFileName,harMeanRecomb,
							centralRegionPops,savedReps,repsPerAnnealStep);
	ClineFitInitializer testParamInitializer(leftEnd,rightEnd,
							testCenter,testWidth,testThetaL,testThetaR,testZL,testZR,
  							testDiseq,harMeanRecomb,testMeanLowScore,testMeanHighScore,
  							covMatrixListType,testCov,IcovOnOff,
//  							testLeftVariance,testCenterVariance,testRightVariance,
  							paramsTurnedOn,paramIdentities,dataFileName,outputFileName,harMeanRecomb,
							centralRegionPops,savedReps,repsPerAnnealStep);
	ClineFitInitializer devParamInitializer(leftEnd,rightEnd,
							centerDev,wDev,thetaDev,thetaDev,zDev,zDev,
  							diseqDev,harMeanRecomb,bottomTopDevList,bottomTopDevList,
  							covMatrixListType,devCovList,IcovOnOff,
//  							varianceDevList,varianceDevList,varianceDevList,
  							paramsTurnedOn,paramIdentities,dataFileName,outputFileName,harMeanRecomb,
							centralRegionPops,savedReps,repsPerAnnealStep);
	ClineFitInitializer minParamInitializer(leftEnd,rightEnd,
							centerMin,wMin,thetaMin,thetaMin,zMin,zMin,
  							diseqMin,harMeanRecomb,meanLowScoreMin,meanLowScoreMin,
  							covMatrixListType,minCovList,IcovOnOff,
//  							traitVarMin,traitVarMin,traitVarMin,
  							paramsTurnedOn,paramIdentities,dataFileName,outputFileName,harMeanRecomb,
							centralRegionPops,savedReps,repsPerAnnealStep);
	ClineFitInitializer maxParamInitializer(leftEnd,rightEnd,
							centerMax,wMax,thetaMax,thetaMax,zMax,zMax,
  							diseqMax,harMeanRecomb,meanHighScoreMax,meanHighScoreMax,
  							covMatrixListType,maxCovList,IcovOnOff,
//  							traitVarMax,traitVarMax,traitVarMax,
  							paramsTurnedOn,paramIdentities,dataFileName,outputFileName,harMeanRecomb,
							centralRegionPops,savedReps,repsPerAnnealStep);



	long savedRepsTemp = savedReps;
	long repsPerAnnealStepTemp = 50 * varyingparams;
	int repsBetweenSavesTemp = 5 * varyingparams;

	if(interface && ! testingFitToGivenShape){
		cout<<"\tThe numerical estimation algorithm will first try to narrow down to a best estimate \
of cline shape parameters, then it will sample the parameter space to determine the support limits.  \
You can use the default settings for this or override them.  In general, higher values increase the time \
that the program runs, but also give more reliable estimates.  The more parameters being estimated in the \
model, the higher these numbers should be.  But for quick, qualitative looks at cline shape while you get \
the feel of your data, low settings may be all you need."<<endl;
		cout<<"\tThe current settings are:"<<endl<<endl;
		int settingMHparams=true;
		while(settingMHparams){
			cout<<"\tNarrowing for initial best estimate (the \'annealing\' phase):"<<endl;
			cout<<"\t\t1\t"<<repsPerAnnealStepTemp<<"\tparameter tries per annealing step (at least 50/parameter)"<<endl;
			cout<<"\tSampling for support limits:"<<endl;
			cout<<"\t\t2\t"<<savedRepsTemp<<"\treplicates saved in garnering support (at least 2000)"<<endl;
			cout<<"\t\t3\t"<<repsBetweenSavesTemp<<"\treplicates between saves (at least 5/parameter)"<<endl;
			cout<<"\tThe last helps ensure that successive saved replicates will be independent."<<endl;
			cout<<"\tEnter the parameter number you would like to adjust.  Enter \'done' when satisfied; \
enter \'default\' to restore the default settings."<<endl;
			cout<<"? Parameter to adjust (1/2/3/default/done)?"<<endl;
			char rawparamword[10]; rawparamword[0]='\0';
			cin>>rawparamword;
			aString paramword(rawparamword);
			paramword.CullLeadingWhiteSpace();
			paramword.CullTrailingWhiteSpace();
			if(paramword=="done" || paramword=="DONE"){
					settingMHparams=false;}
				else if (paramword=="default" || paramword == "DEFAULT"){
					savedRepsTemp = savedReps;
					repsPerAnnealStepTemp = repsPerAnnealStep;
					repsBetweenSavesTemp = repsBetweenSaves;
					}
				else if (paramword.isNumber()){
					int param = paramword.toInt();
					switch(param){
						case 1:
							cout<<"? reps per step = "<<endl;
							cin>>repsPerAnnealStepTemp;
							if(repsPerAnnealStepTemp<1){repsPerAnnealStepTemp=1;}
							break;
						case 2:
							cout<<"? saved reps = "<<endl;
							cin>>savedRepsTemp;
							if(savedRepsTemp<1){savedRepsTemp=1;}
							break;
						case 3:
							cout<<"? reps between saves = "<<endl;
							cin>>repsBetweenSavesTemp;
							if(repsBetweenSavesTemp<0){repsBetweenSavesTemp=0;}
							break;
						default:
							cout<<"Didn\'t understand command "<<param<<".  Please try again..."<<endl;
							break;
						}//switch
					}
				else{
					cout<<"Didn\'t understand command "<<paramword<<".  Please try again..."<<endl;}
			}//end while
					
		savedReps = savedRepsTemp;
		repsPerAnnealStep = repsPerAnnealStepTemp;
		repsBetweenSaves = repsBetweenSavesTemp;

		}//interface

	if(genotypicDataUsed || mtDNAdataUsed){
		switch (paramsToVary){
			case 2:
//				FitClineCW(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
//					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				FitClineCWMulti(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
			case 4:
//				FitClineCWTB(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
///					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				FitClineCWTBMulti(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
			case 6:
//				FitClineCWTlTrZlZr(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
//					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				FitClineCWTlTrZlZrMulti(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
			case 7:
//				FitClineCWTlTrZlZrD(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
//					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				FitClineCWTlTrZlZrDMulti(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
			case 8:
//				FitClineCWTlTrZlZrTB(ran, dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
//					savedReps, repsPerAnnealStep,seed);
				FitClineCWTlTrZlZrTBMulti(ran, dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
			case 9:
//				FitClineCWTlTrZlZrTBD(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
//					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				FitClineCWTlTrZlZrTBDMulti(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
			default:
				break;
			}//end switch paramsToVary
		}//end if codominant or mtDNA data
	if(morphologicalDataUsed){
		switch (paramsToVary){
			case 8:
				FitClineCWTlTrZlZrTB(ran, dataSetUsed,startingParamInitializer,
					testParamInitializer,devParamInitializer,
					minParamInitializer,maxParamInitializer,
					outputFileName,testingFitToGivenShape,
					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
				break;
//			case 9:
//				FitClineCWTlTrZlZrTBD(ran, (ClineGeneticData*) dataSetUsed,startingParamInitializer,
//					testParamInitializer,devParamInitializer,
//					minParamInitializer,maxParamInitializer,
//					outputFileName,testingFitToGivenShape,
//					savedReps, repsPerAnnealStep,seed,repsBetweenSaves);
//				break;
			default:
				break;
			}//end switch paramsToVary
		}//end if morph data

/*

//	ClineShapeCWParamRec initialParams(Icenter, Iw);
//	ClineShapeCWParamRec paramDeviations(centerDev, wDev);
//	ClineShapeCWParamRec lowerParamLimits(centerBottom, wBottom);
//	ClineShapeCWParamRec upperParamLimits(centerTop, wTop);//wTop is 2 x length of transect
	ClineShapeCWTlTrParamRec initialParams(Icenter, Iw,IthetaL,IthetaR, IzL,IzR,leftEnd,rightEnd);
	ClineShapeCWTlTrParamRec paramDeviations(centerDev,wDev,thetaDev,thetaDev,zDev,zDev);
	ClineShapeCWTlTrParamRec lowerParamLimits(centerMin, wMin,thetaMin,thetaMin,zMin,zMin);
	ClineShapeCWTlTrParamRec upperParamLimits(centerMax, wMax,thetaMax,thetaMax,zMax,zMax);//wTop is 2 x length of transect
	lowerParamLimits.xL_=leftEnd;
	upperParamLimits.xR_=rightEnd;

//	ObjList<ParamRec> savedParamList;


//	ObjList<ParamRec> bestParamsLoc;
	MetropolisHastings findMaxLikelihood(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	findMaxLikelihood.SetLikelihoodFunction(MaxLikeDevFromCline);
	findMaxLikelihood.Anneal(ran, 100.0, 10.0, 0.9);
	findMaxLikelihood.SampleForSupportLimits(ran, 1.0, 2000);
	findMaxLikelihood.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.cat(".sum");
	findMaxLikelihood.PrintParamEstimates(outFileSummmaryName, 1);//1 for tab separators
	findMaxLikelihood.PrintParamEstimates(cout, 0);//0 for no tab separators
*/

	delete dataSetUsed; dataSetUsed=0;
	delete watch; watch=0;
	cout<<"All done"<<endl;
	return 0;
	}//end main





/* code that confirms that the determinant and matrix inverse calculations are correct
	this gives the same answers as mathematica
	
	aMatrix<double>* qqq = IdentityMatrix(1.0,3);
	aMatrix<double>& qq = *qqq;
	qq[0][0] = 1.03253;			qq[0][1] = -0.908253;		qq[0][2] = 0.801075;
	qq[1][0] = -0.908253;		qq[1][1] = 1.01286;			qq[1][2] = 0.0409374;
	qq[2][0] = 0.801075;		qq[2][1] = 0.0409374;		qq[2][2] = 2.85559;
	qq.Print();
	double det0 = qq.Determinant();
	cout<<"det: "<<det0<<endl;
	aMatrix<double>* qqqinv = qq.Inverse();
	qqqinv->Print();
	double det1 = qqqinv->Determinant();
	cout<<"det: "<<det1<<endl;
	delete qqqinv; delete qqq;
	return 0;
*/




